/*
 * Dynamic Particle System Library (DPSl)
 * Version:  1.0
 *
 * Universidade do Minho 2000/2001
 * Licenciatura em Engenharia de Sistemas e Informatica - LESI
 *
 * Jose Miguel Araujo Ferreira (jmaferreira@hotmail.com)
 * Pedro Gabriel Ferreira      (pgabiferreira@portugalmail.pt)
 * Rui Manuel A. R. S. Guerra  (ruiguerra@hotmail.com)
 *
 */

#include "dps_demo.h"

#define TIME_INCREMENT 0.3


DPSshandle pset;
DPSshandle fset;
DPSshandle oset;
DPShandle explosao = NULL;

float sim_time = 0;

float simulation_time() {
  return sim_time;
}


unsigned int live_particles() {
  return dpsLiveParticles(pset);
}



void fireworks(DPShandle p, DPSshandle ps, DPSshandle fs, DPSshandle os) {
  DPSvector3f pos;
  int i;
  

  dpsParticlePosition3fv(pos, p);
  dpsColor3f(0.5f, 0.5f, 0.5f);
  dpsRGBADev(0.5f, 0.5f, 0.5f, 0.0f);
  dpsNewParticleSystem(ps);
  for(i=0; i< 60; i++) {
    dpsDieFunc(DPS_NULL);
    dpsPosition3f(pos[0] + get_rand(-0.2, 0.2), 
                  pos[1] + get_rand(-0.2, 0.2),
                  pos[2] + get_rand(-0.2, 0.2));
    dpsVelocity3f(0, 0, 0);
    dpsNewParticle();
  }  
  dpsEndParticleSystem();
  if(explosao) dpsRemoveForce(fs, explosao);
  dpsNewForceSystem(fs);
    explosao = dpsNewExplosion(pos[0], pos[1], pos[2], 0, 5, 5, 1, 0);
  dpsEndForceSystem();
  dpsDieFunc(fireworks);
}




void dps_init() {
  pset = dpsGenParticleSystem(1);
  fset = dpsGenForceSystem(1);
  oset = dpsGenObjectSystem(1);
  

  dpsNewForceSystem(fset);
    dpsNewGravity(0.0f, -0.098f, 0.0f);
    dpsNewKillOld(30.0f, DPS_FALSE);
  dpsEndForceSystem();



  dpsColor3f(0.3f, 0.3f, 0.3f);
  dpsRGBADev(0.2f, 0.2f, 0.2f, 0.0f);
  dpsDieFunc(fireworks);

  dpsNewObjectSystem(oset);
    dpsNewCone(0.0, -15.0, 0.0, 
               0.0, -10.0, 0.001,
               0.5, 5.0, 
               DPS_SOURCE, .02, 2.5, 0.3);

  dpsEndObjectSystem();


  dpsTimeIncrementf(TIME_INCREMENT);
}








void dps_iterate() {
  dpsIterateParticles(pset, fset, oset);
  sim_time += TIME_INCREMENT;
}





void dps_display() {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  dpsDrawParticleSystem(DPS_LINES | DPS_COLOR, pset);
}


