/*
 * Dynamic Particle System Library (DPSl)
 * Version:  1.0
 *
 * Universidade do Minho 2000/2001
 * Licenciatura em Engenharia de Sistemas e Informatica - LESI
 *
 * Jose Miguel Araujo Ferreira (jmaferreira@hotmail.com)
 * Pedro Gabriel Ferreira      (pgabiferreira@portugalmail.pt)
 * Rui Manuel A. R. S. Guerra  (ruiguerra@hotmail.com)
 *
 */

#include "force.h"



// alloc's memory for a force
DPSforce* dpsNewForceAlloc() {
  return (DPSforce*)malloc(sizeof(DPSforce));
}

/*********************     alloc's memory for a explosion... *********************************/
DPSexplosion* dpsNewExplosionAlloc() {
    return (DPSexplosion*)malloc(sizeof(DPSexplosion));
}

DPSfollow* dpsNewFollowAlloc() {
    return (DPSfollow*)malloc(sizeof(DPSfollow));
}

DPSgravitate* dpsNewGravitateAlloc() {
    return (DPSgravitate*)malloc(sizeof(DPSgravitate));
}

DPSgravity* dpsNewGravityAlloc() {
    return (DPSgravity*)malloc(sizeof(DPSgravity));
}

DPSjet* dpsNewJetAlloc() {
    return (DPSjet*)malloc(sizeof(DPSjet));
}

DPSspeedLimit* dpsNewSpeedLimitAlloc() {
    return (DPSspeedLimit*)malloc(sizeof(DPSspeedLimit));
}

DPStargetColor* dpsNewTargetColorAlloc() {
    return (DPStargetColor*)malloc(sizeof(DPStargetColor));
}

DPStargetVelocity* dpsNewTargetVelocityAlloc() {
    return (DPStargetVelocity*)malloc(sizeof(DPStargetVelocity));
}

DPSkillOld* dpsNewKillOldAlloc() {
    return (DPSkillOld*)malloc(sizeof(DPSkillOld));
}

DPSdamping* dpsNewDampingAlloc() {
    return (DPSdamping*)malloc(sizeof(DPSdamping));
}

DPSorbitLine* dpsNewOrbitLineAlloc() {
    return (DPSorbitLine*)malloc(sizeof(DPSorbitLine));
}

DPSorbitPoint* dpsNewOrbitPointAlloc() {
    return (DPSorbitPoint*)malloc(sizeof(DPSorbitPoint));
}

DPSvortex* dpsNewVortexAlloc() {
    return (DPSvortex*)malloc(sizeof(DPSvortex));
}

/*********************    FIM        (alloc's memory for a explosion...)  *********************************/


// retorna um nodo do tipo DPSforce para por na lista de forcas
DPSforce* dpsNewExplosion8f(DPSfloat x, DPSfloat y, DPSfloat z,
                            DPSfloat velocity, 
		            DPSfloat magnitude, 
		            DPSfloat stdev, 
		            DPSfloat epsilon,
		            DPSfloat age) {
    
    DPSexplosion *exp = dpsNewExplosionAlloc();
    DPSforce     *frc = dpsNewForceAlloc();
    
    exp -> position[0] = x;
    exp -> position[1] = y;
    exp -> position[2] = z;
    exp -> velocity    = velocity;
    exp -> magnitude   = magnitude;
    exp -> stdev       = stdev;
    exp -> epsilon     = epsilon;
    exp -> age         = age;
  
    frc -> type  = DPS_EXPLOSION;
    frc -> force = exp;
    frc -> dt    = 0.0f;
    
    return frc;
}


DPSforce* dpsNewFollow3f(DPSfloat magnitude, DPSfloat epsilon, DPSfloat max_radius) {
    
    DPSfollow     *fol = dpsNewFollowAlloc();
    DPSforce      *frc = dpsNewForceAlloc();
  

    fol -> magnitude   = magnitude;
    fol -> epsilon     = epsilon;
    fol -> max_radius  = max_radius;
  
    frc -> type  = DPS_FOLLOW;
    frc -> force = fol;
    frc -> dt    = 0.0f;

    return frc;
}




DPSforce* dpsNewGravitate3f(DPSfloat magnitude, DPSfloat epsilon, DPSfloat max_radius) {
    DPSgravitate  *gra = dpsNewGravitateAlloc();
    DPSforce      *frc = dpsNewForceAlloc();
  

    gra -> magnitude   = magnitude;
    gra -> epsilon     = epsilon;
    gra -> max_radius  = max_radius;
  
    frc -> type  = DPS_GRAVITATE;
    frc -> force = gra;
    frc -> dt    = 0.0f; 

    return frc;
}


DPSforce* dpsNewGravity3f(DPSfloat x, DPSfloat y, DPSfloat z) {

    DPSgravity    *gra = dpsNewGravityAlloc();
    DPSforce      *frc = dpsNewForceAlloc();
  
    gra -> direction[0]   = x;
    gra -> direction[1]   = y;
    gra -> direction[2]   = z;

  
    frc -> type  = DPS_GRAVITY;
    frc -> force = gra;
    frc -> dt    = 0.0f;

    return frc;
}

DPSforce* dpsNewJet9f(DPSfloat cx, DPSfloat cy, DPSfloat cz,
		      DPSfloat ax, DPSfloat ay, DPSfloat az,
		      DPSfloat magnitude, 
		      DPSfloat epsilon,
		      DPSfloat max_radius){

    DPSjet     *jet = dpsNewJetAlloc();
    DPSforce   *frc = dpsNewForceAlloc();
  
    jet -> center[0] = cx;
    jet -> center[1] = cy;
    jet -> center[2] = cz;
    jet -> acc[0]      = ax;
    jet -> acc[1]      = ay;
    jet -> acc[2]      = az;
    jet -> magnitude   = magnitude;
    jet -> epsilon     = epsilon;
    jet -> max_radius  = max_radius;
  
    frc -> type  = DPS_JET;
    frc -> force = jet;
    frc -> dt    = 0.0f;
    
    return frc;
}

DPSforce* dpsNewSpeedLimit2f(DPSfloat min, DPSfloat max){
    DPSspeedLimit  *spd = dpsNewSpeedLimitAlloc();
    DPSforce       *frc = dpsNewForceAlloc();
    
    
    spd -> min_speed   = min;
    spd -> max_speed   = max;
    
    frc -> type  = DPS_SPEEDLIMIT;
    frc -> force = spd;
    frc -> dt    = 0.0f; 
    
    return frc;
}
DPSforce* dpsNewTargetColor5f(DPSfloat c1, DPSfloat c2, DPSfloat c3, DPSfloat alpha, DPSfloat scale){
    DPStargetColor *tgc = dpsNewTargetColorAlloc();
    DPSforce       *frc = dpsNewForceAlloc();
    
    
    tgc -> color[0]   = c1;
    tgc -> color[1]   = c2;
    tgc -> color[2]   = c3;
    tgc -> color[3]   = alpha;
    tgc -> scale      = scale;
    
    frc -> type  = DPS_TARGETCOLOR;
    frc -> force = tgc;
    frc -> dt    = 0.0f; 
    
    return frc;
}
/*
DPSforce* dpsNewTargetSize6f(DPSfloat sz0, DPSfloat sz1, DPSfloat sz2, DPSfloat sc0, DPSfloat sc1, DPSfloat sc2){
    DPStargetColor *siz = dpsNewTargetColorAlloc();
    DPSforce       *frc = dpsNewForceAlloc();
        
    siz -> size[0]   = sz0;
    siz -> size[1]   = sz1;
    siz -> size[2]   = sz2;

    siz -> scale[0]  = sc0;
    siz -> scale[1]  = sc1;
    siz -> scale[2]  = sc2;
    
    frc -> type  = DPS_TARGETCOLOR;
    frc -> force = siz;
    frc -> dt    = 0.0f; 
    
    return frc;
}
*/
DPSforce* dpsNewTargetVelocity4f(DPSfloat v0, DPSfloat v1, DPSfloat v2, DPSfloat scale){
    DPStargetVelocity *vel = dpsNewTargetVelocityAlloc();
    DPSforce          *frc = dpsNewForceAlloc();
    
    
    vel -> velocity[0]   = v0;
    vel -> velocity[1]   = v1;
    vel -> velocity[2]   = v2;
    vel -> scale      = scale;
    
    frc -> type  = DPS_TARGETVELOCITY;
    frc -> force = vel;
    frc -> dt    = 0.0f; 
    
    return frc;
}

DPSforce*  dpsNewKillOld2f(DPSfloat age_limit, DPSboolean  kill_less_than){
    DPSkillOld    *kil = dpsNewKillOldAlloc();
    DPSforce      *frc = dpsNewForceAlloc();

    kil -> age_limit = age_limit;
    kil -> kill_less_than = kill_less_than;
    
    frc -> type  = DPS_KILLOLD;
    frc -> force = kil;
    frc -> dt    = 0.0f; 

    return frc;
}


DPSforce* dpsNewdamping5f(DPSfloat d0, DPSfloat d1, DPSfloat d2, DPSfloat vlowSqr, DPSfloat vhighSqr){
    DPSdamping    *dam = dpsNewDampingAlloc();
    DPSforce      *frc = dpsNewForceAlloc();

    dam -> damping[0] = d0;
    dam -> damping[1] = d1;
    dam -> damping[2] = d2;
    dam -> vlowSqr    = vlowSqr;
    dam -> vhighSqr   = vhighSqr;
    
    frc -> type  = DPS_DAMPING;
    frc -> force = dam;
    frc -> dt    = 0.0f; 
    
    return frc;
}





DPSforce* dpsNewOrbitLine9f(DPSfloat p0, 
			 DPSfloat p1,  
			 DPSfloat p2, 
			 DPSfloat axis0, 
			 DPSfloat axis1,
			 DPSfloat axis2, 
			 DPSfloat magnitude, DPSfloat epsilon, DPSfloat max_radius){
    DPSorbitLine    *orl = dpsNewOrbitLineAlloc();
    DPSforce      *frc = dpsNewForceAlloc();

    orl -> p[0] = p0;
    orl -> p[1] = p1;
    orl -> p[2] = p2;
    orl -> axis[0] = axis0;
    orl -> axis[1] = axis1;
    orl -> axis[2] = axis2;
    orl -> magnitude   = magnitude;
    orl -> epsilon     = epsilon;
    orl -> max_radius  = max_radius;
    
    frc -> type  = DPS_ORBITLINE;
    frc -> force = orl;
    frc -> dt    = 0.0f; 
    
    return frc;

}

DPSforce* dspNewOrbitPoint6f(DPSfloat c0, DPSfloat c1,DPSfloat c2, DPSfloat magnitude, DPSfloat epsilon, DPSfloat max_radius){
    DPSorbitPoint    *orl = dpsNewOrbitPointAlloc();
    DPSforce         *frc = dpsNewForceAlloc();

    orl -> center[0] = c0;
    orl -> center[1] = c1;
    orl -> center[2] = c2;
    orl -> magnitude   = magnitude;
    orl -> epsilon     = epsilon;
    orl -> max_radius  = max_radius;
    
    frc -> type  = DPS_ORBITPOINT;
    frc -> force = orl;
    frc -> dt    = 0.0f; 
    
    return frc;
}


DPSforce* dpsNewVortex9f(DPSfloat p0, 
			 DPSfloat p1,  
			 DPSfloat p2, 
			 DPSfloat axis0, 
			 DPSfloat axis1,
			 DPSfloat axis2, 
			 DPSfloat magnitude, DPSfloat epsilon, DPSfloat max_radius){

    DPSorbitLine  *orl = dpsNewOrbitLineAlloc();
    DPSforce      *frc = dpsNewForceAlloc();

    orl -> p[0] = p0;
    orl -> p[1] = p1;
    orl -> p[2] = p2;
    orl -> axis[0] = axis0;
    orl -> axis[1] = axis1;
    orl -> axis[2] = axis2;
    orl -> magnitude   = magnitude;
    orl -> epsilon     = epsilon;
    orl -> max_radius  = max_radius;
    
    frc -> type  = DPS_VORTEX;
    frc -> force = orl;
    frc -> dt    = 0.0f; 
    
    return frc;

}









/*****************************************        INICIO                *****************************************************/
void explosion(DPSforce *force, GList* particles, DPSfloat dt) {
    DPSfloat radius = ((DPSexplosion*)force->force)->velocity * ((DPSexplosion*)force->force)->age;
    DPSfloat magdt  = ((DPSexplosion*)force->force)->magnitude * dt;
    DPSfloat one_over_sigma = 1.0f / ((DPSexplosion*)force->force)->stdev;
    DPSfloat in_exp = -0.5f * one_over_sigma * one_over_sigma; // humm
    DPSfloat out_exp = DPS_ONE_OVER_SQRT_2_PI * one_over_sigma;
      
    DPSvector3f dir;    
    DPSfloat    dist_sqr;
    DPSfloat    dist;
    DPSfloat    dist_wave_sqr;
    DPSfloat    gd;
    GList* p    = particles;
      

    while(p) {
        dpsSubVect(dir, ((DPSparticle*)p->data)->position, ((DPSexplosion*)force->force)->position);

        dpsSqrLengthVect(&dist_sqr, dir);
        dist = sqrtf(dist_sqr);
        dist_wave_sqr = (radius - dist);
        dist_wave_sqr *= dist_wave_sqr;
 
        gd = exp(dist_wave_sqr * in_exp) * out_exp;
      
        dpsMulVectConst(dir, dir, gd * magdt / (dist * (dist_sqr +((DPSexplosion*)force->force)->epsilon)));
        dpsAddVect(((DPSparticle*)p->data)->velocity, ((DPSparticle*)p->data)->velocity, dir);
        p=g_list_next(p);
    }
    ((DPSexplosion*)force->force)->age+=dt;      
} 


void gravity(DPSforce *force, GList* particles, DPSfloat dt){
    DPSvector3f ddir;
    GList* p;
    dpsMulVectConst(ddir, ((DPSgravity*)force->force)->direction, dt);
	
    for(p = particles; p; p=g_list_next(p)){
	// Step velocity with acceleration
        dpsAddVect(((DPSparticle*)p->data)->velocity, ((DPSparticle*)p->data)->velocity, ddir);
    }

}


void follow(DPSforce *force, GList* particles, DPSfloat dt){
  DPSvector3f tohim;
  DPSfloat tohimlenSqr;
  float magdt = ((DPSfollow*)force->force)->magnitude * dt;
  float max_radiusSqr = ((DPSfollow*)force->force)->max_radius * ((DPSfollow*)force->force)->max_radius;
  GList* p2; 
  GList* p = particles;
  DPSfloat epsilon =  ((DPSfollow*)force->force)->epsilon;
    
 	
  if(max_radiusSqr < DPS_MAXFLOAT)
      {	    	     
	  for(p2 = g_list_next(p); p2;  p = g_list_next(p), p2 = g_list_next(p2))
	      {		
		  // Accelerate toward the particle after me in the list.

		  dpsSubVect(tohim,((DPSparticle*)p2->data)->position , ((DPSparticle*)p->data)->position);//tohim = p1 - p0
		  dpsSqrLengthVect(&tohimlenSqr, tohim);
			
		  if(tohimlenSqr < max_radiusSqr)
		      {
			  // Compute force exerted between the two bodies
			  dpsMulVectConst(tohim, tohim, (magdt / (sqrtf(tohimlenSqr) * (tohimlenSqr + epsilon))));
			  dpsAddVect(((DPSparticle*)p->data)->velocity, ((DPSparticle*)p->data)->velocity, tohim);

		      }
	      }
      }
  else
      {
	  for(p2 = g_list_next(p); p2;  p = g_list_next(p), p2 = g_list_next(p2))
	      {		
		  // Accelerate toward the particle after me in the list.

		  dpsSubVect(tohim,((DPSparticle*)p2->data)->position , ((DPSparticle*)p->data)->position);//tohim = p1 - p0
		  dpsSqrLengthVect(&tohimlenSqr, tohim);
			
	       
		  // Compute force exerted between the two bodies
		  dpsMulVectConst(tohim, tohim, (magdt / (sqrtf(tohimlenSqr) * (tohimlenSqr + epsilon))));
		  dpsAddVect(((DPSparticle*)p->data)->velocity, ((DPSparticle*)p->data)->velocity, tohim);
		    
			
	      }
      }
}

void gravitate(DPSforce *force, GList* particles, DPSfloat dt){
    DPSfloat magdt =  ((DPSgravitate*)force->force)->magnitude * dt;
    DPSfloat max_radiusSqr = ((DPSgravitate*)force->force)->max_radius *  ((DPSgravitate*)force->force)->max_radius;
    DPSfloat epsilon =  ((DPSgravitate*)force->force)->epsilon;
    GList* pj;
    GList* pi;
	
    if(max_radiusSqr < DPS_MAXFLOAT)
	{
	    for(pi = particles; pi;  pi = g_list_next(pi))	     
		{
			
		    // Add interactions with other particles
		    for(pj = g_list_next(pi); pj;  pj = g_list_next(pj))	
			{
			    DPSvector3f tohim;
			    DPSfloat tohimlenSqr;
			    DPSvector3f acc;
			    dpsSubVect(tohim, ((DPSparticle*)pj->data)->position, ((DPSparticle*)pi->data)->position);// tohim = p1 - p0
			    dpsSqrLengthVect(&tohimlenSqr, tohim);
				
			    if(tohimlenSqr < max_radiusSqr)
				{
				    // Compute force exerted between the two bodies
				    dpsMulVectConst(acc, tohim, (magdt / (sqrtf(tohimlenSqr) * (tohimlenSqr + epsilon))));
					
				    dpsAddVect(((DPSparticle*)pi->data)->velocity,((DPSparticle*)pi->data)->velocity, acc);
				    dpsSubVect(((DPSparticle*)pj->data)->velocity,((DPSparticle*)pj->data)->velocity, acc);
				}
			}
		}
	}
    else
	{

	    for(pi = particles; pi;  pi = g_list_next(pi))	     
		{
			
		    // Add interactions with other particles
		    for(pj = g_list_next(pi); pj;  pj = g_list_next(pj))	
			{
			    DPSvector3f tohim;
			    DPSfloat tohimlenSqr;
			    DPSvector3f acc;
			    dpsSubVect(tohim, ((DPSparticle*)pj->data)->position, ((DPSparticle*)pi->data)->position);// tohim = p1 - p0
			    dpsSqrLengthVect(&tohimlenSqr, tohim);

			    // Compute force exerted between the two bodies
			    dpsMulVectConst(acc, tohim, (magdt / (sqrtf(tohimlenSqr) * (tohimlenSqr + epsilon))));
					
			    dpsAddVect(((DPSparticle*)pi->data)->velocity,((DPSparticle*)pi->data)->velocity, acc);
			    dpsSubVect(((DPSparticle*)pj->data)->velocity,((DPSparticle*)pj->data)->velocity, acc);
				
			}
		}

	}
}

void jet(DPSforce *force, GList* particles, DPSfloat dt){
    DPSfloat magdt = ((DPSjet*)force->force)->magnitude * dt;
    DPSfloat max_radiusSqr = ((DPSjet*)force->force)->max_radius *  ((DPSjet*)force->force)->max_radius;
    GList* p = particles;
	
    if(max_radiusSqr < DPS_MAXFLOAT)
	{
	    for(p = g_list_next(p); p;  p = g_list_next(p)) 
		{  
		    
		    // Figure direction to particle.
		    DPSvector3f dir;
		    DPSfloat rSqr;
		    DPSfloat accel;
		    dpsSubVect(dir, ((DPSparticle*)p->data)->position, ((DPSjet*)force->force)->center);
			
		    // Distance to jet (force drops as 1/r^2)
		    // Soften by epsilon to avoid tight encounters to infinity
		    dpsSqrLengthVect(&rSqr, dir);

			
/* 		    if(rSqr < max_radiusSqr) */
/* 			{ */
/* 			    acc.Generate(accel); */
				
/* 				// Step velocity with acceleration */
/* 			    m.vel += accel * (magdt / (rSqr + epsilon)); */
/* 			} */

		}
	}
    else
	{
	    for(p = g_list_next(p); p;  p = g_list_next(p)) 
		{ 
			
		    // Figure direction to particle.
		    DPSvector3f dir;
		    DPSfloat rSqr;
		    dpsSubVect(dir, ((DPSparticle*)p->data)->position, ((DPSjet*)force->force)->center);
			
		    // Distance to jet (force drops as 1/r^2)
		    // Soften by epsilon to avoid tight encounters to infinity
		    dpsSqrLengthVect(&rSqr, dir);
			
/* 		    pVector accel; */
/* 		    acc.Generate(accel); */
			
/* 		    // Step velocity with acceleration */
/* 		    m.vel += accel * (magdt / (rSqr + epsilon)); */
		}
	}
}

void speedLimit(DPSforce *force, GList* particles, DPSfloat dt){
    DPSfloat min_speed = ((DPSspeedLimit*)force->force)->min_speed;
    DPSfloat max_speed = ((DPSspeedLimit*)force->force)->max_speed;     
    DPSfloat min_sqr = min_speed* min_speed;
    DPSfloat max_sqr = max_speed* max_speed;	

    DPSfloat sSqr;
    DPSfloat s;
    GList* p;

    for(p = particles; p;  p = g_list_next(p)){
   
	dpsSqrLengthVect(&sSqr, ((DPSparticle*)p->data)->velocity);

	if(sSqr<min_sqr && sSqr)
	    {
		s =  (min_speed/sqrtf(sSqr));
		dpsMulVectConst( ((DPSparticle*)p->data)->velocity, ((DPSparticle*)p->data)->velocity,s);
	    }
	else if(sSqr>max_sqr)
	    {
		s =  (max_speed/sqrtf(sSqr));
		dpsMulVectConst( ((DPSparticle*)p->data)->velocity, ((DPSparticle*)p->data)->velocity, s);
	    }
    }
}



void targetColor(DPSforce *force, GList* particles, DPSfloat dt){
    DPSfloat scaleFac = ((DPStargetColor*)force->force)->scale * dt;     
    GList* p;
    DPSvector4f tmp;

    for(p = particles; p;  p = g_list_next(p)){
	dpsSubVect(tmp, ((DPStargetColor*)force->force)->color, ((DPSparticle*)p->data)->color);
	dpsMulVectConst(tmp, tmp, scaleFac);
	dpsAddVect(((DPSparticle*)p->data)->color, ((DPSparticle*)p->data)->color, tmp);
    }
}



void targetVelocity(DPSforce *force, GList* particles, DPSfloat dt){ 
    DPSfloat scaleFac = ((DPStargetVelocity*)force->force)->scale * dt;     
    GList* p;
    DPSvector4f tmp;

    for(p = particles; p;  p = g_list_next(p)){
	dpsSubVect(tmp, ((DPStargetVelocity*)force->force)->velocity, ((DPSparticle*)p->data)->velocity);
	dpsMulVectConst(tmp, tmp, scaleFac);
	dpsAddVect(((DPSparticle*)p->data)->velocity, ((DPSparticle*)p->data)->velocity, tmp);
    }
}








void killold(DPSforce *force, GList** particles, DPSfloat dt, DPSsets sets){      
    GList *t, *p;

    for(p = *particles; p;  p = g_list_next(p)) {

      if(((DPSparticle*)p->data)->age < 
            ((DPSkillOld*)force->force)->age_limit && 
             ((DPSkillOld*)force->force)->kill_less_than || 
               ((DPSparticle*)p->data)->age> 
	         ((DPSkillOld*)force->force)->age_limit && 
		   !((DPSkillOld*)force->force)->kill_less_than) {

        // executar a funcao die quando uma particula morre
        if(((DPSparticle*)p->data)->die)  
          ((DPSparticle*)p->data)->die((DPSparticle*)p->data, 
                                        sets.particles,
                                        sets.forces,
                                        sets.objects);



        t = p;
        p = g_list_next(p);
        *particles = g_list_remove_link(*particles, t);

      }
      
    }
}











void damping(DPSforce *force, GList* particles, DPSfloat dt){    
    GList* p;
    DPSvector3f one = {1,1,1};
    DPSvector3f scale;
    DPSfloat vSqr;

    // This is important if dt is != 1.       scale = one - ((one - damping) * dt)
    dpsSubVect(scale, one, dpsMulVectConst(scale,dpsSubVect(scale, one, ((DPSdamping*)force->force)->damping), dt));

    for(p = particles; p;  p = g_list_next(p)){
	dpsSqrLengthVect(&vSqr, ((DPSparticle*)p->data)->velocity);


	if(vSqr >= ((DPSdamping*)force->force)->vlowSqr && vSqr <= ((DPSdamping*)force->force)->vhighSqr){
	   ((DPSparticle*)p->data) -> velocity[0] *= scale[0];
	   ((DPSparticle*)p->data) -> velocity[1] *= scale[1];
	   ((DPSparticle*)p->data) -> velocity[2] *= scale[2];
	}
    }
}













void orbitLine(DPSforce *force, GList* particles, DPSfloat dt){
    DPSfloat magdt =  ((DPSorbitLine*)force->force)->magnitude * dt;
    DPSfloat max_radiusSqr = ((DPSorbitLine*)force->force)->max_radius *  ((DPSorbitLine*)force->force)->max_radius;
    DPSfloat epsilon =  ((DPSorbitLine*)force->force)->epsilon;
    GList* p;
    DPSvector3f f, w, into;
    DPSfloat rSqr;

    if(max_radiusSqr < DPS_MAXFLOAT)
	{
	    for(p = particles; p;  p = g_list_next(p))	     
		{
		    // Figure direction to particle from base of line.
		    // f = position - p
		    dpsSubVect(f, ((DPSparticle*)p->data)->position, ((DPSorbitLine*)force->force)->p);
		    // w = axis * (f * axis)
		    dpsProdVect(w,((DPSorbitLine*)force->force)->axis, dpsProdVect(w , f, ((DPSorbitLine*)force->force)->axis));
		    // Direction from particle to nearest point on line.
		    dpsSubVect(into, w, f);


		    // Distance to line (force drops as 1/r^2, normalize by 1/r)
		    // Soften by epsilon to avoid tight encounters to infinity
		    dpsSqrLengthVect(&rSqr, into);


		    if(rSqr < max_radiusSqr){
			// Step velocity with acceleration
			dpsMulVectConst(into, into, (magdt / (sqrtf(rSqr) + (rSqr + epsilon))));
			dpsAddVect(((DPSparticle*)p->data)->velocity, ((DPSparticle*)p->data)->velocity, into);
		    }
		}
	}
    else
	{
	    // Removed because it causes pipeline stalls.
	    for(p = particles; p;  p = g_list_next(p))	     
		{
		    // Figure direction to particle from base of line.
		    // f = position - p
		    dpsSubVect(f, ((DPSparticle*)p->data)->position, ((DPSorbitLine*)force->force)->p);
		    // w = axis * (f * axis)
		    dpsProdVect(w,((DPSorbitLine*)force->force)->axis, dpsProdVect(w , f, ((DPSorbitLine*)force->force)->axis));
		    // Direction from particle to nearest point on line.
		    dpsSubVect(into, w, f);

		    // Distance to line (force drops as 1/r^2, normalize by 1/r)
		    // Soften by epsilon to avoid tight encounters to infinity
		    dpsSqrLengthVect(&rSqr, into);

		    // Step velocity with acceleration
		    dpsMulVectConst(into, into, (magdt / (sqrtf(rSqr) + (rSqr + epsilon))));
		    dpsAddVect(((DPSparticle*)p->data)->velocity, ((DPSparticle*)p->data)->velocity, into); 
		}
	}
}





void orbitPoint(DPSforce *force, GList* particles, DPSfloat dt){

}




void vortex(DPSforce *force, GList* particles, DPSfloat dt){
    DPSfloat magdt =  ((DPSvortex*)force->force)->magnitude * dt;
    DPSfloat max_radiusSqr = ((DPSvortex*)force->force)->max_radius *  ((DPSvortex*)force->force)->max_radius;
    DPSfloat epsilon =  ((DPSvortex*)force->force)->epsilon;
    GList* p;
    DPSvector3f offset, offnorm, w, u, v;
    DPSfloat rSqr, r, theta, s, c, axisProj;
    int i;

    if(max_radiusSqr < DPS_MAXFLOAT)
	{

	    for(p = particles; p;  p = g_list_next(p))	     
		{
		    // Vector from tip of vortex
		    //dpsSubVect(offset, ((DPSparticle*)p->data)->pos, center);
			
		    // Compute distance from particle to tip of vortex.
		    dpsSqrLengthVect(&rSqr, dpsSubVect(offset, ((DPSparticle*)p->data)->position,((DPSvortex*)force->force)->center));
			
		    // Don't do anything to particle if too close or too far.
		    if(rSqr > max_radiusSqr)
			continue;
			
		    r = sqrtf(rSqr);
			
		    // Compute normalized offset vector.
		    dpsDivVectConst(offnorm, offset, r);
			
		    // Construct orthogonal vector frame in which to rotate
		    // transformed point around origin
		    dpsProdScalVect(&axisProj, offnorm, ((DPSvortex*)force->force)->axis); // offnorm . axis
		   			
		    // Components of offset perpendicular and parallel to axis
		    dpsMulVectConst(w, ((DPSvortex*)force->force)->axis, axisProj); // parallel component
		    dpsSubVect(u, offnorm, w); // perpendicular component
			
			// Perpendicular component completing frame:

		    // ATENCAO ATENCAO ATENCAO   tou a tentar fazer:   v(axis ^ u);
		    dpsProdVect(v, ((DPSvortex*)force->force)->axis, u);
		    // Figure amount of rotation
		    // Resultant is (cos theta) u + (sin theta) v
		    theta = magdt / (rSqr + epsilon);
		    s = sinf(theta);
		    c = cosf(theta);
			

		    //ESTOU A TENTAR FAZER ISTO:  offset = (u * c + v * s + w) * r;

		    dpsAddVect(u, dpsMulVectConst(u, u ,c), dpsMulVectConst(v, v ,s));
		    dpsAddVect(u, u, w);
		    dpsMulVectConst(offset, u, r);

		    // Translate back to object space
		    dpsAddVect(((DPSparticle*)p->data)->position, offset, ((DPSvortex*)force->force)->center);
		}
	}
    else
	{

	    // Removed because it causes pipeline stalls.
	    for(p = particles; p;  p = g_list_next(p))	     
		{
		    printf("OK2\n");
		    // Vector from tip of vortex
		    //dpsSubVect(offset, ((DPSparticle*)p->data)->pos, center);
			
		    // Compute distance from particle to tip of vortex.
		    dpsSqrLengthVect(&rSqr, dpsSubVect(offset, ((DPSparticle*)p->data)->position,((DPSvortex*)force->force)->center));
			
		    r = sqrtf(rSqr);
			
		    // Compute normalized offset vector.
		    dpsDivVectConst(offnorm, offset, r);
			
		    // Construct orthogonal vector frame in which to rotate
		    // transformed point around origin
		    dpsProdScalVect(&axisProj, offnorm, ((DPSvortex*)force->force)->axis); // offnorm . axis
			
		    // Components of offset perpendicular and parallel to axis
		    dpsMulVectConst(w, ((DPSvortex*)force->force)->axis, axisProj); // parallel component
		    dpsSubVect(u, offnorm, w); // perpendicular component
			
			// Perpendicular component completing frame:
		    // ATENCAO ATENCAO ATENCAO       NAO SEI FAZER ISTO    v(axis ^ u);


		    // Figure amount of rotation
		    // Resultant is (cos theta) u + (sin theta) v
		    theta = magdt / (rSqr + epsilon);
		    s = sinf(theta);
		    c = cosf(theta);
			
		    //ESTOU A TENTAR FAZER ISTO:  offset = (u * c + v * s + w) * r;

		    dpsAddVect(u, dpsMulVectConst(u, u ,c), dpsMulVectConst(v, v ,s));
		    dpsAddVect(u, u, w);
		    dpsMulVectConst(offset, u, r);

		    // Translate back to object space
		    dpsAddVect(((DPSparticle*)p->data)->position, offset, ((DPSvortex*)force->force)->center);
		}		
	}
}
/***************************************************        FIM                  ******************************************************/
void dpsApplyForce(DPSforce *force, GList** particles, DPSstate *state, DPSsets sets) {
    switch(force->type) {
    case DPS_EXPLOSION:
	explosion(force, *particles, state->__DPS_STATE_DT);
    break; 
    case DPS_FOLLOW:
	follow(force, *particles, state->__DPS_STATE_DT);
    break; 
    case DPS_GRAVITY: 
	gravity(force, *particles, state->__DPS_STATE_DT);
    break; 
    case DPS_GRAVITATE:
	gravitate(force, *particles, state->__DPS_STATE_DT);
    break;
    case DPS_JET:
	jet(force, *particles, state->__DPS_STATE_DT);
    break;
    case DPS_SPEEDLIMIT:
	speedLimit(force, *particles, state->__DPS_STATE_DT);
    break;
    case DPS_TARGETCOLOR:
	targetColor(force, *particles, state->__DPS_STATE_DT);
    break;
    case DPS_TARGETVELOCITY:
	targetVelocity(force, *particles, state->__DPS_STATE_DT);
    break;
    case DPS_KILLOLD:
	killold(force, particles, state->__DPS_STATE_DT, sets);
    break;
    case DPS_DAMPING:
	damping(force, *particles, state->__DPS_STATE_DT);
    break;
    case DPS_ORBITPOINT:
	orbitLine(force, *particles, state->__DPS_STATE_DT);
    break;
    case DPS_ORBITLINE:
	orbitPoint(force, *particles, state->__DPS_STATE_DT);
    break;
    case DPS_VORTEX:
	vortex(force, *particles, state->__DPS_STATE_DT);
	break;
  }
}







