/*
 * Dynamic Particle System Library (DPSl)
 * Version:  1.0
 *
 * Universidade do Minho 2000/2001
 * Licenciatura em Engenharia de Sistemas e Informatica - LESI
 *
 * Jose Miguel Araujo Ferreira (jmaferreira@hotmail.com)
 * Pedro Gabriel Ferreira      (pgabidf@portugalmail.pt)
 * Rui Manuel A. R. S. Guerra  (ruiguerra@hotmail.com)
 *
 */

#include "object.h"




DPSobject* dpsNewObjectAlloc() {
  return (DPSobject*)malloc(sizeof(DPSobject));
}

DPSsphere* dpsNewSphereAlloc() {
  return (DPSsphere*)malloc(sizeof(DPSsphere));
}

DPStriangle* dpsNewTriangleAlloc() {
  return (DPStriangle*)malloc(sizeof(DPStriangle));
}

DPSrectangle* dpsNewRectangleAlloc() {
  return (DPSrectangle*)malloc(sizeof(DPSrectangle));
}

DPSplane* dpsNewPlaneAlloc() {
  return (DPSplane*)malloc(sizeof(DPSplane));
}


DPScube* dpsNewCubeAlloc() {
  return (DPScube*)malloc(sizeof(DPScube));
}

DPSdisc* dpsNewDiscAlloc() {
  return (DPSdisc*)malloc(sizeof(DPSdisc));
}

DPScylinder* dpsNewCylinderAlloc() {
  return (DPScylinder*)malloc(sizeof(DPScylinder));
}


DPScone* dpsNewConeAlloc() {
  return (DPScone*)malloc(sizeof(DPScone));
}



// retornam todos objectos prontos a inserir nos grupos
DPSobject* dpsNewSphere9f(DPSfloat x, DPSfloat y, DPSfloat z, 
			  DPSfloat ri, DPSfloat ro,
  		          DPSenum action, 
			  DPSfloat var1, DPSfloat var2, DPSfloat var3) {

  DPSsphere *sph = dpsNewSphereAlloc();
  DPSobject *obj = dpsNewObjectAlloc();
  
  sph->position[0] = x;
  sph->position[1] = y;
  sph->position[2] = z;

 
 // inicializacoes das variaveis da esfera
 if(ro > ri){
   sph->radiusIn    = ri;
   sph->radiusOut   = ro;
 }else{
   sph->radiusOut   = ri;
   sph->radiusIn    = ro;
 }
   sph->radiusInSqr  = (sph->radiusIn)  *(sph->radiusIn);
   sph->radiusOutSqr = (sph->radiusOut) *(sph->radiusOut);

  obj->type    = DPS_SPHERE;
  obj->action  = action;
  obj->object  = sph;
  obj->var1    = var1;
  obj->var2    = var2;
  obj->var3    = var3;
  obj->counter = 0.0f; //inicializa o contador
  return obj;
}


DPSobject* dpsNewTriangle13f(DPSfloat x0, DPSfloat y0, DPSfloat z0,
                          DPSfloat x1, DPSfloat y1, DPSfloat z1, 
			  DPSfloat x2, DPSfloat y2, DPSfloat z2,
  		          DPSenum action, 
			  DPSfloat var1, DPSfloat var2, DPSfloat var3){

//inicilizacao de um objecto triangulo
  DPStriangle *tri = dpsNewTriangleAlloc();
  DPSobject   *obj = dpsNewObjectAlloc();
  DPSvector3f  tu, tv, tmp1, tmp2;         //variaveis temporarias
  DPSfloat     usqr, vsqr;                //para calculo intermedios
  //vector director
  tri->position1[0] = x0;
  tri->position1[1] = y0;
  tri->position1[2] = z0;
  //ponto 1
  tri->tp1[0] = x1;
  tri->tp1[1] = y1;
  tri->tp1[2] = z1;
  //ponto 2    
  tri->tp2[0] = x2;
  tri->tp2[1] = y2;
  tri->tp2[2] = z2;

  

 // inicializacoes das variaveis do triangulo
  //vectores direccao do plano do triangulo
  dpsSubVect(tri->u, tri->tp1, tri->position1); 
  dpsSubVect(tri->v, tri->tp2, tri->position1); 
 
  dpsLengthVect(&usqr, tri->u);
  dpsLengthVect(&vsqr, tri->v);
 
  dpsDivVectConst(tu, tri->u, usqr);
  dpsDivVectConst(tv, tri->v, vsqr);
  
  //calcula-se o vector normal aos vectores tu tv 
  dpsProdVect(tmp1, tu, tv); 
  dpsNormalizeVect(tri->position2, tmp1); //calcula o vector de norma unitaria:tmp2
  
  //calcula-se o parametro D da equacao do plano
  //utilizado para o calculo da interseccao
  dpsProdScalVect(&(tri->distance),tri->position1, tri->position2);  
  tri->distance*=-1.0f;

  obj->type    = DPS_TRIANGLE;
  obj->action  = action;
  obj->object  = tri;
  obj->var1    = var1;
  obj->var2    = var2;
  obj->var3    = var3;
  obj->counter = 0.0f;
  
  return obj;
}


DPSobject* dpsNewRectangle13f(DPSfloat x0, DPSfloat y0, DPSfloat z0,
                          DPSfloat x1, DPSfloat y1, DPSfloat z1, 
			  DPSfloat x2, DPSfloat y2, DPSfloat z2,
  		          DPSenum action, 
			  DPSfloat var1, DPSfloat var2, DPSfloat var3){

//inicilizacao de um objecto rectangulo
  DPSrectangle *rec = dpsNewRectangleAlloc();
  DPSobject    *obj = dpsNewObjectAlloc();
  DPSvector3f  tu, tv, tmp1;  //variaveis temporarias
  DPSfloat     usqr, vsqr; //para calculo intermedios
  
  rec->position1[0] = x0;
  rec->position1[1] = y0;
  rec->position1[2] = z0;
  
  rec->u[0]        = x1;
  rec->u[1]        = y1;
  rec->u[2]        = z1;
  
  rec->v[0]        = x2;
  rec->v[1]        = y2;
  rec->v[2]        = z2;

  
 // inicializacoes das variaveis do rectangulo
 
  dpsLengthVect(&usqr, rec->u);
  dpsLengthVect(&vsqr, rec->v);
 
  dpsDivVectConst(tu, rec->u, usqr);
  dpsDivVectConst(tv, rec->v, vsqr);
 
  dpsProdVect(tmp1, tu, tv);    //calcula-se o vector normal aos vectores tu tv
  dpsNormalizeVect(rec->position2, tmp1); //calcula o vector de norma unitaria:tmp2
  
  //calcula-se o parametro D da equacao do plano
  //utilizado para o calculo da interseccao
  dpsProdScalVect(&(rec->distance),rec->position1, rec->position2);  
  rec->distance *= -1.0f;


  obj->type    = DPS_RECTANGLE;
  obj->action  = action;
  obj->object  = rec;
  obj->var1    = var1;
  obj->var2    = var2;
  obj->var3    = var3;
  obj->counter = 0.0f;  //inicializa o contador


  return obj;
}

//inicializacao do objecto plano
DPSobject* dpsNewPlane10f(DPSfloat x0, DPSfloat y0, DPSfloat z0,
                          DPSfloat x1, DPSfloat y1, DPSfloat z1, 
		          DPSenum action, 
			  DPSfloat var1, DPSfloat var2, DPSfloat var3) {

  DPSplane  *pln  = dpsNewPlaneAlloc();
  DPSobject *obj = dpsNewObjectAlloc();
  
  pln->position1[0] = x0;
  pln->position1[1] = y0;
  pln->position1[2] = z0;

  pln->position2[0] = x1;
  pln->position2[1] = y1;
  pln->position2[2] = z1;

  dpsNormalizeVect(pln->position2, pln->position2);
  
  //calculo   da distancia D da eq do plano
  dpsProdScalVect(&(pln->distance), pln->position1, pln->position2);  
  pln->distance*=-1.0f;
  
  obj->type    = DPS_PLANE;
  obj->action  = action;
  obj->object  = pln;
  obj->var1    = var1;
  obj->var2    = var2;
  obj->var3    = var3;
  obj->counter = 0.0f;  //inicializa o contador

  return obj;
}



//inicializacao do objecto cubo

DPSobject* dpsNewCube10f(DPSfloat x0, DPSfloat y0, DPSfloat z0, 
                        DPSfloat x1, DPSfloat y1, DPSfloat z1, 
  		        DPSenum action, 
			DPSfloat var1, DPSfloat var2, DPSfloat var3) {
  
  DPScube   *cub = dpsNewCubeAlloc();
  DPSobject *obj = dpsNewObjectAlloc();

  if(x0 < x1){
    cub->position1[0] = x0; 
    cub->position2[0] = x1;
  }else{
    cub->position1[0] = x1; 
    cub->position2[0] = x0;
  }
  if(y0 < y1){
    cub->position1[1] = y0; 
    cub->position2[1] = y1;
  }else{
    cub->position1[1] = y1; 
    cub->position2[1] = y0;
  }
  if(z0 < z1){
   cub->position1[2] = z0; 
   cub->position2[2] = z1; 
  }else{
   cub->position1[2] = z1; 
   cub->position2[2] = z0; 
  }
  
  obj->type    = DPS_CUBE;
  obj->action  = action;
  obj->object  = cub;
  obj->var1    = var1;
  obj->var2    = var2;
  obj->var3    = var3;
  obj->counter = 0.0f;  //inicializa o contador

  return obj;
  
  
}


//inicializacao de um objecto disco
DPSobject* dpsNewDisc12f(DPSfloat x0, DPSfloat y0, DPSfloat z0, 
                         DPSfloat x1, DPSfloat y1, DPSfloat z1, 
		     	 DPSfloat ri, DPSfloat ro,
  		         DPSenum action, 
			 DPSfloat var1, DPSfloat var2, DPSfloat var3) {
  
  DPSdisc   *dsc = dpsNewDiscAlloc();
  DPSobject *obj = dpsNewObjectAlloc();
  DPSvector3f Vtmp;
  DPSfloat tmp;
  
  dsc->position1[0] = x0;
  dsc->position1[1] = y0;
  dsc->position1[2] = z0;

  dsc->position2[0] = x1;
  dsc->position2[1] = y1;
  dsc->position2[2] = z1;

  dsc->basis[0] = 1.0f;
  dsc->basis[1] = 0.0;
  dsc->basis[2] = 0.0;

  dpsNormalizeVect(dsc->position2, dsc->position2);
  
  if(ri < ro){
    dsc->radiusIn  = ri;
    dsc->radiusOut = ro;
  }else{
    dsc->radiusIn  = ro;
    dsc->radiusOut = ri;
  }   
  //quadrados dos raios
  dsc->radiusInSqr  = dsc->radiusIn  * dsc->radiusIn;
  dsc->radiusOutSqr = dsc->radiusOut * dsc->radiusOut;

  //encontar um vector ortogonal a position2
  dpsProdVect(&tmp, dsc->basis, dsc->position2);
  if(fabs(tmp)>0.999){
    dsc->basis[0] = 0.0;
    dsc->basis[1] = 1.0f;
    dsc->basis[2] = 0.0;
  }
    
  //projectar a componente normal
  //normalizaar calc o prod vectorial e obter o seg. vector ortonormal
  dpsProdScalVect(&tmp, dsc->basis, dsc->position2);
  dpsMulVectConst(Vtmp, dsc->position2, tmp);
  dpsSubVect(dsc->u, dsc->basis, Vtmp);
  dpsNormalizeVect(dsc->u, dsc->u); 
  dpsProdVect(dsc->v, dsc->position2, dsc->u);
  dpsProdScalVect(&(dsc->distance), dsc->position1, dsc->position2);  
  dsc->distance*=-1.0f;
  
  
  obj->type    = DPS_DISC;
  obj->action  = action;
  obj->object  = dsc;
  obj->var1    = var1;
  obj->var2    = var2;
  obj->var3    = var3;
  obj->counter = 0.0f;  //inicializa o contador

  return obj;

}


//inicializacao do objecto cone
DPSobject* dpsNewCone12f(DPSfloat x0, DPSfloat y0, DPSfloat z0, 
                         DPSfloat x1, DPSfloat y1, DPSfloat z1, 
			 DPSfloat ri, DPSfloat ro,
  		         DPSenum action, 
			 DPSfloat var1, DPSfloat var2, DPSfloat var3){
			 
			 
  DPScone   *con = dpsNewConeAlloc();
  DPSobject *obj = dpsNewObjectAlloc();
  DPSvector3f Vtmp; 
  DPSfloat  p212, tmp;
  
  con->position1[0] = x0; //base
  con->position1[1] = y0;
  con->position1[2] = z0;

  con->tmp[0] = x1;  //bico
  con->tmp[1] = y1;
  con->tmp[2] = z1;
  
  //positon2 e'o vector direccao de pos1 ate a outra extremidade 
  // do cone.Pos1 e  centro da base do cone
  dpsSubVect(con->position2, con->tmp, con->position1); 
  if(ri < ro){
    con->radiusIn  = ri;
    con->radiusOut = ro;
  }else{
    con->radiusIn  = ro;
    con->radiusOut = ri;
  }   
  con->radiusOutSqr =(con->radiusOut)*(con->radiusOut);
  
  //dado um qq vector n, diferente de zero
  //calcular 2 vectores ortonormais u e v
  con->n[0] = con->position2[0];
  con->n[1] = con->position2[1];
  con->n[2] = con->position2[2];
  
  dpsSqrLengthVect(&p212, con->n);
  dpsNormalizeVect(con->n, con->n);
  
  //radiusInSqr guarda 1/p2.p2)
  //para verificar se interessa o raio exterior
  con->radiusInSqr = p212 ? 1.0f /p212 : 0.0f;
  //encontrar um vector ortogonal a n
  con->basis[0] = 1.0f;
  con->basis[1] = 0.0;
  con->basis[2] = 0.0;
  
  //encontar um vector ortogonal a n
  dpsProdVect(&tmp, con->basis, con->n);
  if(fabs(tmp)>0.999){
    con->basis[0] = 0.0;
    con->basis[1] = 1.0f;
    con->basis[2] = 0.0;
  }
  
  //projectar a componente normal
  //normalizaar calc o prod vectorial e obter o seg. vector ortonormal
  dpsProdScalVect(&tmp, con->basis, con->n);
  dpsMulVectConst(Vtmp, con->n, tmp);
  dpsSubVect(con->u, con->basis, Vtmp);
  dpsNormalizeVect(con->u, con->u); 
  dpsProdVect(con->v, con->n, con->u);
  
  obj->type    = DPS_CONE;
  obj->action  = action;
  obj->object  = con;
  obj->var1    = var1;
  obj->var2    = var2;
  obj->var3    = var3;
  obj->counter = 0.0f;  //inicializa o contador

  return obj;		 
}


//inicializacao do objecto cilindro
//e'um copia excta do caso cone
DPSobject* dpsNewCylinder12f(DPSfloat x0, DPSfloat y0, DPSfloat z0, 
                             DPSfloat x1, DPSfloat y1, DPSfloat z1, 
			     DPSfloat ri, DPSfloat ro,
  		             DPSenum action, 
			     DPSfloat var1, DPSfloat var2, DPSfloat var3){
			 
			 
  DPScylinder   *cyl = dpsNewCylinderAlloc();
  DPSobject     *obj = dpsNewObjectAlloc();
  DPSvector3f Vtmp; 
  DPSfloat  p212, tmp;
  
  cyl->position1[0] = x0;
  cyl->position1[1] = y0;
  cyl->position1[2] = z0;

  cyl->tmp[0] = x1;
  cyl->tmp[1] = y1;
  cyl->tmp[2] = z1;
  
  //positon2 e'o vector direccao de pos1 ate a outra extremidade 
  // do cone.Pos1 e  centro da base do cone
  dpsSubVect(cyl->position2, cyl->tmp, cyl->position1); 
  if(ri < ro){
    cyl->radiusIn  = ri;
    cyl->radiusOut = ro;
  }else{
    cyl->radiusIn  = ro;
    cyl->radiusOut = ri;
  }   
  cyl->radiusOutSqr =(cyl->radiusOut)*(cyl->radiusOut);
  
  //dado um qq vector n, diferente de zero
  //calcular 2 vectores ortonormais u e v
  cyl->n[0] = cyl->position2[0];
  cyl->n[1] = cyl->position2[1];
  cyl->n[2] = cyl->position2[2];
  
  dpsSqrLengthVect(&p212, cyl->n);
  dpsNormalizeVect(cyl->n, cyl->n);
  
  //radiusInSqr guarda 1/p2.p2)
  //para verificar se interessa o raio exterior
  cyl->radiusInSqr = p212 ? 1.0f /p212 : 0.0f;
  //encontrar um vector ortogonal a n
  cyl->basis[0] = 1.0f;
  cyl->basis[1] = 0.0;
  cyl->basis[2] = 0.0;
  
  //encontar um vector ortogonal a n
  dpsProdVect(&tmp, cyl->basis, cyl->n);
  if(fabs(tmp)>0.999){
    cyl->basis[0] = 0.0;
    cyl->basis[1] = 1.0f;
    cyl->basis[2] = 0.0;
  }
  
  //projectar a componente normal
  //normalizaar calc o prod vectorial e obter o seg. vector ortonormal
  dpsProdScalVect(&tmp, cyl->basis, cyl->n);
  dpsMulVectConst(Vtmp, cyl->n, tmp);
  dpsSubVect(cyl->u, cyl->basis, Vtmp);
  dpsNormalizeVect(cyl->u, cyl->u); 
  dpsProdVect(cyl->v, cyl->n, cyl->u);
  
  obj->type    = DPS_CYLINDER;
  obj->action  = action;
  obj->object  = cyl;
  obj->var1    = var1;
  obj->var2    = var2;
  obj->var3    = var3;
  obj->counter = 0.0f;  //inicializa o contador

  return obj;

  
  
  			 
}















/**************************************************************************/
/**** FUNCOES QUE CALCULAM UM DADO PONTO SE ENCONTRA NO INTERIOR        ***/
/**** DO OOBJECTO                                                       ***/
/**************************************************************************/

// verifica se um dado ponto se encontra dentro da area da esfera
// i.e, entre o raio interior eo exterior
DPSboolean isWithinSphere(DPSfloat x, DPSfloat y, DPSfloat z, DPSsphere *sph){
  DPSvector3f Point, Vtmp;
  DPSfloat rsqr;   
  Point[0] = x;
  Point[1] = y;
  Point[2] = z;
  dpsSubVect(Vtmp, Point, sph->position);
  dpsSqrLengthVect(&rsqr, Vtmp);
  return ((rsqr <= sph->radiusOutSqr) && (rsqr >= sph->radiusInSqr));
      
}

//verifica se um dado ponto se encontra dentro do cubo
DPSboolean isWithinCube(DPSfloat x, DPSfloat y, DPSfloat z, DPScube *cub){
  return !((x < cub->position1[0]) || (x > cub->position2[0]) ||
           (y < cub->position1[1]) || (y > cub->position2[1]) ||           
	   (z < cub->position1[2]) || (z > cub->position2[2]));
}



DPSboolean isWithinPlane(DPSfloat x1, DPSfloat y1, DPSfloat z1, 
                         DPSfloat x2, DPSfloat y2, DPSfloat z2, 
                         DPSplane *pln){

  DPSvector3f PosAfter, PosBefore;
  DPSfloat distold, distnew;
  
  PosBefore[0] = x1;
  PosBefore[1] = y1;
  PosBefore[2] = z1;

  PosAfter[0] = x2;
  PosAfter[1] = y2;
  PosAfter[2] = z2;

 
  //positon2 contem o vectro normal ao plano (A,B,C da eq do plano)
  //distold e dist new : posicoes em relacao ao plano antes e depois
  // da iteraccao dist(n,plano)=n*p+D
  //distnace guarda -n*p,ou seja D da eq do plano

  dpsProdScalVect(&distold, PosBefore, pln->position2);
  distold+=pln->distance;

  dpsProdScalVect(&distnew, PosAfter, pln->position2);
  distnew+=pln->distance;

  return (distold * distnew <= 0.0f);
}


DPSboolean isWithinDisc(DPSfloat x1, DPSfloat y1, DPSfloat z1, 
                        DPSfloat x2, DPSfloat y2, DPSfloat z2, 
			DPSfloat vx, DPSfloat vy, DPSfloat vz, 
                        DPSdisc *dsc){

  DPSvector3f PosAfter, PosBefore, Vel, Vtmp, Phit, Offset;
  DPSfloat distold, distnew, nv, t, rad;
  
  PosBefore[0] = x1;
  PosBefore[1] = y1;
  PosBefore[2] = z1;

  PosAfter[0] = x2;
  PosAfter[1] = y2;
  PosAfter[2] = z2;

  Vel[0] = vx;
  Vel[1] = vy;
  Vel[2] = vz;

 
  //positon2 contem o vectro normal ao plano (A,B,C da eq do plano)
  //distold e dist new : posicoes em relacao ao plano antes e depois
  // da iteraccao dist(n,plano)=n*p+D
  //distnace guarda -n*p,ou seja D da eq do plano

  dpsProdScalVect(&distold, PosBefore, dsc->position2);
  distold+=dsc->radiusOutSqr;

  dpsProdScalVect(&distnew, PosAfter, dsc->position2);
  distnew+=dsc->radiusOutSqr;

  if (distold * distnew >= 0.0f) return 0;
  
  dpsProdScalVect(&nv, dsc->position2, Vel);
  t = - 1.0f*distold /nv;
  
  dpsMulVectConst(Vtmp, Vel, t);
  dpsAddVect(Phit, PosBefore, Vtmp);
  
  dpsSubVect(Offset, Phit, dsc->position1);
  dpsSqrLengthVect(&rad, Offset);
  
  return (rad < dsc->radiusOutSqr && rad > dsc->radiusInSqr);

}











DPSboolean isWithinTriangle(DPSfloat x1, DPSfloat y1, DPSfloat z1, 
                            DPSfloat x2, DPSfloat y2, DPSfloat z2, 
                            DPStriangle *tri){

  DPSvector3f Pnext, Pbefore;
  DPSfloat distold, distnew;  
  
  Pnext[0] = x2;
  Pnext[1] = y2;
  Pnext[2] = z2;
  
  Pbefore[0] = x1;
  Pbefore[1] = y1;
  Pbefore[2] = z1;

      
    //positon2 contem o vector normal ao plano (A,B,C da eq do plano)
    //distold e dist new : posicoes em relacao ao plano antes e depois
    // da iteraccao dist(n,plano)=n*p+D
    //distnace guarda -n*p,ou seja D da eq do plano
    dpsProdScalVect(&distold, Pbefore, tri->position2);
    distold+=tri->distance;

    dpsProdScalVect(&distnew, Pnext , tri->position2);
    distnew+=tri->distance;
    
    return(distold * distnew <= 0);
   
}

DPSboolean isWithinRectangle(DPSfloat x1, DPSfloat y1, DPSfloat z1, 
                             DPSfloat x2, DPSfloat y2, DPSfloat z2, 
                             DPSrectangle *rec){

  DPSvector3f Pnext, Pbefore;
  DPSfloat distold, distnew;  
  
  Pnext[0] = x2;
  Pnext[1] = y2;
  Pnext[2] = z2;
  
  Pbefore[0] = x1;
  Pbefore[1] = y1;
  Pbefore[2] = z1;

      
    //positon2 contem o vector normal ao plano (A,B,C da eq do plano)
    //distold e dist new : posicoes em relacao ao plano antes e depois
    // da iteraccao dist(n,plano)=n*p+D
    //distnace guarda -n*p,ou seja D da eq do plano
    dpsProdScalVect(&distold, Pbefore, rec->position2);
    distold+=rec->distance;

    dpsProdScalVect(&distnew, Pnext , rec->position2);
    distnew+=rec->distance;
    
    return(distold * distnew <= 0);
   
}







//verifica se um ponto esta dentro do cone
DPSboolean isWithinCone(DPSfloat x, DPSfloat y, DPSfloat z, DPScone *con){
  DPSvector3f X, Xrad, Pos,Vtmp;
  DPSfloat dist, tmp, rsqr;
  Pos[0] = x;
  Pos[1] = y;
  Pos[2] = z;
  //sendo position2 o vector direccao da base ate ao topo do cilindro 
  //X e'o vector diferenca da base ate ao ponto a testar
  //X.p2 : dist=> projeccao ao longo do eixo x
  //rad = X -dist*p2 => proj de x ao longo da base

  dpsSubVect(X, Pos, con->position1); 
  //verificar dist axial
  dpsProdScalVect(&tmp, con->position2, X);
  dist = tmp * (con->radiusInSqr);
    if(dist < 0.0f || dist > 1.0f)
      return 0;
        
  dpsMulVectConst(Vtmp, con->position2, dist);
  //componente radial de x
  dpsSubVect(Xrad, X, Vtmp); 
  dpsSqrLengthVect(&rsqr, Xrad);
  
  return (rsqr <= ((dist * (con->radiusOut))*(dist * (con->radiusOut))) &&
          rsqr >= ((dist * (con->radiusIn))* (dist * (con->radiusIn))));


}

//verifica se um ponto esta dentro do cilindro
DPSboolean isWithinCylinder(DPSfloat x, DPSfloat y, DPSfloat z, DPScylinder *cyl){
  DPSvector3f X, Xrad, Pos,Vtmp;
  DPSfloat dist, tmp, rsqr;
  Pos[0] = x;
  Pos[1] = y;
  Pos[2] = z;
  //sendo position2 o vector direccao da base ate ao topo do cilindro 
  //X e'o vector diferenca da base ate ao ponto a testar
  //X.p2 : dist=> projeccao ao longo do eixo x
  //rad = X -dist*p2 => proj de x ao longo da base

  dpsSubVect(X, Pos, cyl->position1); 
  //verificar dist axial
  dpsProdScalVect(&tmp, cyl->position2, X);
  dist = tmp * (cyl->radiusInSqr);
    if(dist < 0.0f || dist > 1.0f)
      return 0;
        
  dpsMulVectConst(Vtmp, cyl->position2, dist);
  //componente radial de x
  dpsSubVect(Xrad, X, Vtmp); 
  dpsSqrLengthVect(&rsqr, Xrad);

  return (rsqr <= cyl->radiusOutSqr && rsqr >= (cyl->radiusIn) * (cyl->radiusIn));

}









/***********************************************************************/
/*** FUNCOES QUE GERAM  UM PONTO DENTRO DE UM OBJECTO *********************/
/** estas funcoes apenas retornam uma posicao no objecto         *********/
/************************************************************************/

DPSfloat* dpsGenCubePos(DPSvector3f Pos,
                        DPSfloat x0, DPSfloat y0, DPSfloat z0, 
                        DPSfloat x1, DPSfloat y1, DPSfloat z1){
			
  
  // escalar e multiplicar por [0,1] para ficar dentro do cubo
  Pos[0] = x0 + ( x1 - x0) * dpsRand48();			
  Pos[1] = y0 + ( y1 - y0) * dpsRand48();			
  Pos[2] = z0 + ( z1 - z0) * dpsRand48();			
  return Pos;
}

//gera ponto dentro do triangulo
DPSfloat* dpsGenTrianglePos(DPSvector3f Pos,
                            DPSfloat x0, DPSfloat y0, DPSfloat z0, 
                            DPSfloat x1, DPSfloat y1, DPSfloat z1,
			    DPSfloat x2, DPSfloat y2, DPSfloat z2){
				
  DPSvector3f Position1, u, v;
  DPSfloat    r1, r2;
  
  r1= dpsRand48();
  r2= dpsRand48();
  
  Position1[0] = x0;
  Position1[1] = y0;
  Position1[2] = z0;
  //vector normal U
  u[0] = x1 - x0;
  u[1] = y1 - y0;
  u[2] = z1 - z0;
  //vector normal V
  v[0] = x2 - x0;
  v[1] = y2 - y0;
  v[2] = z2 - z0;
  
  if(r1 +r2 < 1.0f){
    Pos[0] = Position1[0] + u[0] * r1 + v[0] *r2;
    Pos[1] = Position1[1] + u[1] * r1 + v[1] *r2;
    Pos[2] = Position1[2] + u[2] * r1 + v[2] *r2;
  }else{
    Pos[0] = Position1[0] + u[0] * (1.0f - r1) + v[0] * (1.0f - r2);
    Pos[1] = Position1[1] + u[1] * (1.0f - r1) + v[1] * (1.0f - r2);
    Pos[2] = Position1[2] + u[2] * (1.0f - r1) + v[2] * (1.0f - r2);
  }
  return Pos;
}

DPSfloat* dpsGenRectanglePos(DPSvector3f Pos,
                             DPSfloat x0, DPSfloat y0, DPSfloat z0, 
                             DPSfloat x1, DPSfloat y1, DPSfloat z1,
			     DPSfloat x2, DPSfloat y2, DPSfloat z2){
  DPSvector3f Position1, u, v;

  Position1[0] = x0;
  Position1[1] = y0;
  Position1[2] = z0;
  
  u[0]        = x1;
  u[1]        = y1;
  u[2]        = z1;
  
  v[0]        = x2;
  v[1]        = y2;
  v[2]        = z2;

  Pos[0] = Position1[0] + u[0] * dpsRand48() + v[0] * dpsRand48();
  Pos[1] = Position1[1] + u[1] * dpsRand48() + v[1] * dpsRand48();
  Pos[2] = Position1[2] + u[2] * dpsRand48() + v[2] * dpsRand48();
  
  return Pos;
}


//retorna apenas um ponto do rectangulo : pouco util
DPSfloat* dpsGenPlanePos(DPSvector3f Pos,
                         DPSfloat x0, DPSfloat y0, DPSfloat z0){
  
  Pos[0] = x0;
  Pos[1] = y0;
  Pos[2] = z0;
  return Pos;
}



DPSfloat* dpsGenSpherePos(DPSvector3f Pos,
                          DPSfloat x0, DPSfloat y0, DPSfloat z0,
                          DPSfloat ri, DPSfloat ro ){
			  
  DPSvector3f Position1, randv, halfv;
  DPSfloat rin, rout;
  
  Position1[0] = x0;
  Position1[1] = y0;
  Position1[2] = z0;
  if(ri < ro){
    rout = ro;
    rin  = ri;
  }else{
    rin  = ro;
    rout = ri;
  }

  dpsHalfVect(halfv);
  dpsRandVect(randv);
  dpsSubVect(Pos, randv, halfv);
  dpsNormalizeVect(Pos, Pos);
  
  if(rin == rout){
    Pos[0] = Position1[0] + Pos[0] * rout;
    Pos[1] = Position1[1] + Pos[1] * rout;
    Pos[2] = Position1[2] + Pos[2] * rout;
  }else{
    Pos[0] = Position1[0] + Pos[0] * (rin + dpsRand48() * (rout - rin));
    Pos[1] = Position1[1] + Pos[1] * (rin + dpsRand48() * (rout - rin));
    Pos[2] = Position1[2] + Pos[2] * (rin + dpsRand48() * (rout - rin));
  }
  return Pos;
}



//funcao pouco eficiente uma vez que invoca a funcao newDisc
DPSfloat* dpsGenDiscPos(DPSvector3f Pos,
                        DPSfloat x0, DPSfloat y0, DPSfloat z0,
                        DPSfloat x1, DPSfloat y1, DPSfloat z1, 
                        DPSfloat ri, DPSfloat ro ){
  DPSvector3f Vtmpx, Vtmpy; 
  DPSfloat x, y, r, theta; 

  DPSobject* object = dpsNewDisc12f(x0, y0, z0, 
                                    x1, y1, z1, 
		                    ri, ro,
  		                    0.0, 0.0, 0.0, 0.0);
	
		      
  //angulo em torno da normal		      
  theta = dpsRand48() * 2.0f * ((float)M_PI);
  //distancia do centro
  r = ((DPSdisc*)object->object)->radiusIn + dpsRand48()  *
  (((DPSdisc*)object->object)->radiusOut - ((DPSdisc*)object->object)->radiusIn);
  
  x = r * ((DPSfloat)dpsCos(theta / M_PI * 180.0f));
  y = r * ((DPSfloat)dpsSin(theta / M_PI * 180.0f));
  		      
  
  dpsMulVectConst(Vtmpx, ((DPSdisc*)object->object)->u, x);
  dpsMulVectConst(Vtmpy, ((DPSdisc*)object->object)->v, y);
  dpsAddVect(Vtmpx, Vtmpx, Vtmpy);
  dpsAddVect(Pos, ((DPSdisc*)object->object)->position1, Vtmpx);
  return Pos;
  
}


DPSfloat* dpsGenConePos(DPSvector3f Pos,
                        DPSfloat x0, DPSfloat y0, DPSfloat z0,
                        DPSfloat x1, DPSfloat y1, DPSfloat z1, 
                        DPSfloat ri, DPSfloat ro ){
  DPSvector3f Vtmpx, Vtmpy, Vtmp2; 
  DPSfloat x, y, r, theta, dist;


  DPSobject* object = dpsNewCone12f(x0, y0, z0, 
                                    x1, y1, z1, 
		                    ri, ro,
  		                    0.0, 0.0, 0.0, 0.0);

  //distancia entre o topo e a base	
  dist  =  dpsRand48();		      
  //angulo em torno da normal		      
  theta = dpsRand48() * 2.0f * ((float)M_PI);
  //distancia do centro
  r = ((DPScone*)object->object)->radiusIn + dpsRand48()  *
  (((DPScone*)object->object)->radiusOut - ((DPScone*)object->object)->radiusIn);
  
  x = r * ((DPSfloat)dpsCos(theta / M_PI * 180.0f));
  y = r * ((DPSfloat)dpsSin(theta / M_PI * 180.0f));
  		      
  //escalar raios ao longo dos eixos
  x *= dist;
  y *= dist;
  
  
  dpsMulVectConst(Vtmpx, ((DPScone*)object->object)->u, x);
  dpsMulVectConst(Vtmpy, ((DPScone*)object->object)->v, y);
  dpsMulVectConst(Vtmp2, ((DPScone*)object->object)->position2, dist);
  dpsAddVect(Vtmpx, Vtmpx, Vtmpy);
  dpsAddVect(Vtmp2, Vtmp2, Vtmpx);
  dpsAddVect(Pos, ((DPScone*)object->object)->position1, Vtmp2);
  return Pos;
  
}


DPSfloat* dpsGenCylinderPos(DPSvector3f Pos,
                        DPSfloat x0, DPSfloat y0, DPSfloat z0,
                        DPSfloat x1, DPSfloat y1, DPSfloat z1, 
                        DPSfloat ri, DPSfloat ro ){
  DPSvector3f Vtmpx, Vtmpy, Vtmp2; 
  DPSfloat x, y, r, theta, dist;


  DPSobject* object = dpsNewCylinder12f(x0, y0, z0, 
                                        x1, y1, z1, 
		                        ri, ro,
  		                        0.0, 0.0, 0.0, 0.0);

  //distancia entre o topo e a base	
  dist  =  dpsRand48();		      
  //angulo em torno da normal		      
  theta = dpsRand48() * 2.0f * ((float)M_PI);
  //distancia do centro
  r = ((DPScylinder*)object->object)->radiusIn + dpsRand48()  *
  (((DPScylinder*)object->object)->radiusOut - ((DPScylinder*)object->object)->radiusIn);

  
  x = r * ((DPSfloat)dpsCos(theta / M_PI * 180.0f));
  y = r * ((DPSfloat)dpsSin(theta / M_PI * 180.0f));
  		      
    
  dpsMulVectConst(Vtmpx, ((DPScylinder*)object->object)->u, x);
  dpsMulVectConst(Vtmpy, ((DPScylinder*)object->object)->v, y);
  dpsMulVectConst(Vtmp2, ((DPScylinder*)object->object)->position2, dist);
  dpsAddVect(Vtmpx, Vtmpx, Vtmpy);
  dpsAddVect(Vtmp2, Vtmp2, Vtmpx);
  dpsAddVect(Pos, ((DPScylinder*)object->object)->position1, Vtmp2);
  return Pos;
  
}

  
  








/****************************************************************************/
/****** FIM DAS FUNCOES DE GERACAO DE PONTOS     ****************************/
/****************************************************************************/


/**************************************************************************/
/**** FUNCOES QUE CALCULAM AS COLISOES COM OS DIFERENTES OBJECTOS       ***/
/**************************************************************************/

//funcao que calcula a colisao com um plano
void dpsCollidePlane(DPSobject *object, GList* particles, DPSfloat dt, DPSsets sets){
  GList* p    = particles;
  DPSvector3f Pnext, Vn, Vt, Vtmp;
  DPSfloat distold, distnew, nmag, tmp,n2;
  
  for(;p; p=g_list_next(p)) { //para cada particula
    //verificar se a proxima posicao da particula
    //ja se encontra dentro do objecto
    dpsMulVectConst(Vtmp,((DPSparticle*)p->data)->velocity , dt);
    dpsAddVect(Pnext,((DPSparticle*)p->data)->position, Vtmp);
  
    //positon2 contem o vectro normal ao plano (A,B,C da eq do plano)
    //distold e dist new : posicoes em relacao ao plano antes e depois
    // da iteraccao dist(n,plano)=n*p+D
    //distnace guarda -n*p,ou seja D da eq do plano
    dpsProdScalVect(&tmp,((DPSparticle*)p->data)->position ,((DPSplane*)object->object)->position2);
    distold = tmp + ((DPSplane*)object->object)->distance;
    dpsProdScalVect(&tmp, Pnext ,((DPSplane*)object->object)->position2);
    distnew = tmp + ((DPSplane*)object->object)->distance;
  
    if(distold * distnew >= 0)  continue;

    // executar funcao de colisao se esta estiver definida
    if(((DPSparticle*)p->data)->collide)  
      ((DPSparticle*)p->data)->collide((DPSparticle*)p->data, 
                                        sets.particles,
                                        sets.forces,
                                        sets.objects);

    
    //calcular componente Normal e Tangencial do vector velocidade 
    dpsProdScalVect(&nmag,((DPSparticle*)p->data)->velocity ,((DPSplane*)object->object)->position2);
  
    dpsMulVectConst(Vn,((DPSplane*)object->object)->position2,nmag);
    dpsSubVect(Vt,((DPSparticle*)p->data)->velocity , Vn); 
  
    //calcula a velocidade da particula apos o choque
    //nao aplica fricao se a velocidade tangencial for inferior
    // a velocidade de cutoff
    if(dpsSqrLengthVect(&n2, Vt) <= object->var3){
      dpsMulVectConst(Vn, Vn, object->var2);  
      dpsSubVect(((DPSparticle*)p->data)->velocity, Vt, Vn);
    } else {
      dpsMulVectConst(Vn, Vn, object->var2);  
      dpsMulVectConst(Vt, Vt, object->var1);  
      dpsSubVect(((DPSparticle*)p->data)->velocity, Vt, Vn);
    }
  } 
}


//funcao que calcula a colisao com uma esfera 
void dpsCollideSphere(DPSobject *object, GList* particles, DPSfloat dt, DPSsets sets){
  GList* p    = particles;
  DPSvector3f Pnext, Vn, Vt, Vtmp, Nsup, Vtmp2;
  DPSfloat  nmag, tmp;
  DPSboolean pinside;
  
  
  while(p) { //para cada particula
    //verificar se a proxima posicao da particula
    //ja se encontra dentro do objecto
    dpsMulVectConst(Vtmp,((DPSparticle*)p->data)->velocity , dt);
    dpsAddVect(Pnext,((DPSparticle*)p->data)->position, Vtmp);

    if(isWithinSphere(Pnext[0], Pnext[1], Pnext[2], (DPSsphere*)object->object)){

    // executar funcao de colisao se esta estiver definida
    if(((DPSparticle*)p->data)->collide)  
      ((DPSparticle*)p->data)->collide((DPSparticle*)p->data, 
                                        sets.particles,
                                        sets.forces,
                                        sets.objects);

        //verificar se ja estavamos cada dentro na ultima iteraccao
      pinside = isWithinSphere(((DPSparticle*)p->data)->position[0],
                               ((DPSparticle*)p->data)->position[1], 
			       ((DPSparticle*)p->data)->position[2], 
			       (DPSsphere*)object->object);
      //calcula a normal a'supericie da esfera
      dpsSubVect(Vtmp, ((DPSparticle*)p->data)->position , ((DPSsphere*)object->object)->position);
      dpsNormalizeVect(Nsup, Vtmp);
      //calcula componentes tangenciais e normais da velocidade
      dpsProdScalVect(&nmag,((DPSparticle*)p->data)->velocity, Nsup);
      
      dpsMulVectConst(Vn, Nsup, nmag); //Normal Vn = (Vel.|Nsup|)|Nsup|
      dpsSubVect(Vt,((DPSparticle*)p->data)->velocity , Vn); //Tangente  Vel -Vn


      if(pinside){
        //posicao anterior era dentro.Se acomp. normal da velocidade aponta
	//para dentro entao inverte.Isto repele a particula que de 
	// outra forma ficariam presas na esfer
	  if(nmag <= 0) dpsSubVect(((DPSparticle*)p->data)->velocity, Vt, Vn); 

      } else {
         //posicao anterior da particula era fora da esfera
	 //se a comp. normal de vel. aponta par dentro entao inverte
	 //aplica friccao se Vt>= cutoff; aplica resistencia.
	 //calcula nova velocidade apos choque
	 //nao aplica friccao se vel tangencial inferior a vel de cutoff

	 dpsSqrLengthVect(&tmp, Vt);
	 if(tmp <= object->var3){
	    dpsMulVectConst(Vtmp, Vn, object->var2);
	    dpsSubVect(((DPSparticle*)p->data)->velocity, Vt, Vtmp); 
	 }else{
	    dpsMulVectConst(Vtmp, Vn, object->var2);
	    dpsMulVectConst(Vtmp2, Vt, object->var1);
	    dpsSubVect(((DPSparticle*)p->data)->velocity, Vtmp2, Vtmp); 
	 }
       } 
      } //fecha pinside if 
    p=g_list_next(p);    
  }
}


//colisao de uma particula com um rectangulo

void dpsCollideRectangle(DPSobject *object, GList* particles, DPSfloat dt, DPSsets sets){
  GList* p    = particles;
  DPSvector3f S1,S2, Pnext, Vtmp, Phit, Offset, Vn, Vt;
  DPSfloat wx, wy, wz, xy, xz, ux, uy, uz, vx , vy, vz,
   distold, distnew, t, upos, vpos, nmag, det, n2, tmp;
  
  //calcular a matrix inversa do plano da base
  ux = ((DPSrectangle*)object->object)->u[0];
  uy = ((DPSrectangle*)object->object)->u[1];
  uz = ((DPSrectangle*)object->object)->u[2];
    
  vx = ((DPSrectangle*)object->object)->v[0];
  vy = ((DPSrectangle*)object->object)->v[1];
  vz = ((DPSrectangle*)object->object)->v[2];

  // w = u X v
  wx = uy*vz - uz*vy;
  wy = uz*vx - ux*vz;
  wz = ux*vy - uy*vx;

  det =1/(wz*ux*vy - wz*uy*vx - uz*wx*vy - ux*vz*wy + vz*wx*uy + uz*vx*wy);
  
  S1[0] = vy*wz - vz*wy;
  S1[1] = vz*wx - vx*wz;
  S1[2] = vx*wy - vy*wx;
  dpsMulVectConst(S1, S1, det);
  
  S2[0] = uy*wz - uz*wy;
  S2[1] = uz*wx - ux*wz;
  S2[2] = ux*wy - uy*wx;
  dpsMulVectConst(S2, S2, -1.0f*det);
  
  for(;p; p=g_list_next(p)) { //para cada particula
    dpsMulVectConst(Vtmp,((DPSparticle*)p->data)->velocity, dt);
    dpsAddVect(Pnext,((DPSparticle*)p->data)->position, Vtmp);
    
    //positon2 contem o vectro normal ao plano (A,B,C da eq do plano)
    //distold e dist new : posicoes em relacao ao plano antes e depois
    // da iteraccao dist(n,plano)=n*p+D
    //distnace guarda -n*p,ou seja D da eq do plano
    dpsProdScalVect(&tmp,((DPSparticle*)p->data)->position ,((DPSrectangle*)object->object)->position2);
    distold = tmp + ((DPSrectangle*)object->object)->distance;
    dpsProdScalVect(&tmp, Pnext ,((DPSrectangle*)object->object)->position2);
    distnew = tmp + ((DPSrectangle*)object->object)->distance;
  
    if(distold * distnew >= 0)     continue;
   
    dpsProdScalVect(&tmp,((DPSrectangle*)object->object)->position2,((DPSparticle*)p->data)->velocity);
    t = -1.0f*distold/tmp;
    
    //actual interseccao
    dpsMulVectConst(Vtmp, ((DPSparticle*)p->data)->velocity, t);
    dpsAddVect(Phit, ((DPSparticle*)p->data)->position, Vtmp);
    
    //offset em relacao a origem no plano,p -origem
    dpsSubVect(Offset,Phit,((DPSrectangle*)object->object)->position1);
    
    dpsProdScalVect(&upos, Offset, S1);  
    dpsProdScalVect(&vpos, Offset, S2);
    
    if(upos < 0 || upos > 1 || vpos < 0 || vpos>1)  continue;

    // executar funcao de colisao se esta estiver definida
    if(((DPSparticle*)p->data)->collide)  
      ((DPSparticle*)p->data)->collide((DPSparticle*)p->data, 
                                        sets.particles,
                                        sets.forces,
                                        sets.objects);

    
    dpsProdScalVect(&nmag, ((DPSparticle*)p->data)->velocity, ((DPSrectangle*)object->object)->position2);
    dpsMulVectConst(Vn, ((DPSrectangle*)object->object)->position2, nmag); //veloc. Normal
    dpsSubVect(Vt, ((DPSparticle*)p->data)->velocity, Vn); //Vel tangente
    //calcula a velocidade da particula apos o choque
    //nao aplica fricao se a velocidade tangencial for inferior
    // a velocidade de cutoff
    
    if(dpsSqrLengthVect(&n2, Vt) <= object->var3){
      dpsMulVectConst(Vn, Vn, object->var2);  
      dpsSubVect(((DPSparticle*)p->data)->velocity, Vt, Vn);
    } else {
      dpsMulVectConst(Vn, Vn, object->var2);  
      dpsMulVectConst(Vt, Vt, object->var1);  
      dpsSubVect(((DPSparticle*)p->data)->velocity, Vt, Vn);
    }
  }
}


//definicao da colisao com o objecto triangulo
void dpsCollideTriangle(DPSobject *object, GList* particles, DPSfloat dt, DPSsets sets){
  GList* p    = particles;
  DPSvector3f S1,S2, Pnext, Vtmp, Phit, Offset, Vn, Vt;
  DPSfloat wx, wy, wz, xy, xz, ux, uy, uz, vx , vy, vz,
   distold, distnew, t, upos, vpos, nmag, det, n2, tmp;
  
  //calcular a matrix inversa do plano da base
  ux = ((DPStriangle*)object->object)->u[0];
  uy = ((DPStriangle*)object->object)->u[1];
  uz = ((DPStriangle*)object->object)->u[2];
    
  vx = ((DPStriangle*)object->object)->v[0];
  vy = ((DPStriangle*)object->object)->v[1];
  vz = ((DPStriangle*)object->object)->v[2];

  // w = u X v
  wx = uy*vz - uz*vy;
  wy = uz*vx - ux*vz;
  wz = ux*vy - uy*vx;
    
  det =1/(wz*ux*vy - wz*uy*vx - uz*wx*vy - ux*vz*wy + vz*wx*uy + uz*vx*wy);
  
  S1[0] = vy*wz - vz*wy;
  S1[1] = vz*wx - vx*wz;
  S1[2] = vx*wy - vy*wx;
  dpsMulVectConst(S1, S1, det);
  
  S2[0] = uy*wz - uz*wy;
  S2[1] = uz*wx - ux*wz;
  S2[2] = ux*wy - uy*wx;
  dpsMulVectConst(S2, S2, -1.0f*det);
  
  for(;p; p=g_list_next(p)) { //para cada particula
    dpsMulVectConst(Vtmp,((DPSparticle*)p->data)->velocity, dt);
    dpsAddVect(Pnext,((DPSparticle*)p->data)->position, Vtmp);
    
    //positon2 contem o vector normal ao plano (A,B,C da eq do plano)
    //distold e dist new : posicoes em relacao ao plano antes e depois
    // da iteraccao dist(n,plano)=n*p+D
    //distnace guarda -n*p,ou seja D da eq do plano
    dpsProdScalVect(&tmp,((DPSparticle*)p->data)->position ,((DPStriangle*)object->object)->position2);
    distold = tmp + ((DPStriangle*)object->object)->distance;
    dpsProdScalVect(&tmp, Pnext ,((DPStriangle*)object->object)->position2);
    distnew = tmp + ((DPStriangle*)object->object)->distance;
  
    if(distold * distnew >= 0) continue;
   
    dpsProdScalVect(&tmp,((DPStriangle*)object->object)->position2,((DPSparticle*)p->data)->velocity);
    t = -1.0f*distold/tmp;
    
    //actual interseccao
    dpsMulVectConst(Vtmp, ((DPSparticle*)p->data)->velocity, t);
    dpsAddVect(Phit, ((DPSparticle*)p->data)->position, Vtmp);
    
    //offset em relacao a origem no plano,p -origem
    dpsSubVect(Offset,Phit,((DPStriangle*)object->object)->position1);
    
    dpsProdScalVect(&upos, Offset, S1);  
    dpsProdScalVect(&vpos, Offset, S2);  
    
    if(upos < 0 || vpos <  0 || (upos +vpos) > 1) continue;

    // executar funcao de colisao se esta estiver definida
    if(((DPSparticle*)p->data)->collide)  
      ((DPSparticle*)p->data)->collide((DPSparticle*)p->data, 
                                        sets.particles,
                                        sets.forces,
                                        sets.objects);


    
//    dpsProdScalVect(&nmag, ((DPSparticle*)p->data)->velocity, ((DPStriangle*)object->object)->position2);
    dpsMulVectConst(Vn, ((DPStriangle*)object->object)->position2, tmp); //veloc. Normal
    dpsSubVect(Vt, ((DPSparticle*)p->data)->velocity, Vn); //Vel tangente
    //calcula a velocidade da particula apos o choque
    //nao aplica fricao se a velocidade tangencial for inferior
    // a velocidade de cutoff

    
    if(dpsSqrLengthVect(&n2, Vt) <= object->var3){
      dpsMulVectConst(Vn, Vn, object->var2);  
      dpsSubVect(((DPSparticle*)p->data)->velocity, Vt, Vn);
    } else {
      dpsMulVectConst(Vn, Vn, object->var2);  
      dpsMulVectConst(Vt, Vt, object->var1);  
      dpsSubVect(((DPSparticle*)p->data)->velocity, Vt, Vn);
    }
  }
}








//funcao que calcula a colisao com um disco
void dpsCollideDisc(DPSobject *object, GList* particles, DPSfloat dt, DPSsets sets){
  GList* p    = particles;
  DPSvector3f Pnext, Vn, Vt, Vtmp,Offset, Phit;
  DPSfloat distold, distnew,tmp, n2, routsqr, rinsqr, nv, t, rad;
  
  rinsqr  = ((DPSdisc*)object->object)->radiusIn * ((DPSdisc*)object->object)->radiusIn;
  routsqr = ((DPSdisc*)object->object)->radiusOut * ((DPSdisc*)object->object)->radiusOut;  
  
  
  for(;p; p=g_list_next(p)) { //para cada particula
    //verificar se a proxima posicao da particula
    //ja se encontra dentro do objecto
    dpsMulVectConst(Vtmp,((DPSparticle*)p->data)->velocity , dt);
    dpsAddVect(Pnext,((DPSparticle*)p->data)->position, Vtmp);
  
    //positon2 contem o vectro normal ao plano (A,B,C da eq do plano)
    //distold e dist new : posicoes em relacao ao plano antes e depois
    // da iteraccao dist(n,plano)=n*p+D
    //distnace guarda -n*p,ou seja D da eq do plano
    dpsProdScalVect(&tmp,((DPSparticle*)p->data)->position ,((DPSdisc*)object->object)->position2);
    distold = tmp + ((DPSdisc*)object->object)->distance; //alterado estava antes distance
    dpsProdScalVect(&tmp, Pnext ,((DPSdisc*)object->object)->position2);
    distnew = tmp + ((DPSdisc*)object->object)->distance; //alterado estava antes distance
  
    if(distold * distnew >= 0) continue;

    dpsProdScalVect(&nv, ((DPSdisc*)object->object)->position2 ,((DPSparticle*)p->data)->velocity);     
    t = -1.0f*distold / nv;
    
    //actual interseccao
    dpsMulVectConst(Vtmp, ((DPSparticle*)p->data)->velocity, t);
    dpsAddVect(Phit, ((DPSparticle*)p->data)->position, Vtmp);
    
    //offset em relacao a origem no plano,p -origem
    dpsSubVect(Offset,Phit,((DPSdisc*)object->object)->position1);

    dpsSqrLengthVect(&rad,Offset);
    if(rad > routsqr || rad < rinsqr) continue;

    // executar funcao de colisao se esta estiver definida
    if(((DPSparticle*)p->data)->collide)  
      ((DPSparticle*)p->data)->collide((DPSparticle*)p->data, 
                                        sets.particles,
                                        sets.forces,
                                        sets.objects);
     
    dpsMulVectConst(Vn, ((DPSdisc*)object->object)->position2 , nv);
    dpsSubVect(Vt, ((DPSparticle*)p->data)->velocity, Vn);
    
    //calcula a velocidade da particula apos o choque
    //nao aplica fricao se a velocidade tangencial for inferior
    // a velocidade de cutoff
    if(dpsSqrLengthVect(&n2, Vt) <= object->var3){
      dpsMulVectConst(Vn, Vn, object->var2);  
      dpsSubVect(((DPSparticle*)p->data)->velocity, Vt, Vn);
    } else {
      dpsMulVectConst(Vn, Vn, object->var2);  
      dpsMulVectConst(Vt, Vt, object->var1);  
      dpsSubVect(((DPSparticle*)p->data)->velocity, Vt, Vn);
    }
  } 
}






/**************************************************************************/
/**** FUNCOES QUE CALCULAM O "AVOID"   COM OS DIFERENTES OBJECTOS       ***/
/**************************************************************************/


//funcao que calcula a accao AVOID sobre o objecto ESFERA
void dpsAvoidSphere(DPSobject *object, GList* particles, DPSfloat dt){
  GList* p    = particles;
  DPSfloat vm,v,disc,t,tmp,magdt,rout,routsqr;
  DPSvector3f Vn,L,C,Cnorm,S,Vtmp;


  magdt = object->var2 * dt;
  
  rout = ((DPSsphere*)object->object)->radiusOut;
  routsqr = rout * rout; //quadrado do raio exterior
  
  
  for(;p; p=g_list_next(p)) { //para cada particula
    dpsLengthVect(&vm,((DPSparticle*)p->data)->velocity);
    dpsDivVectConst(Vn, ((DPSparticle*)p->data)->velocity , vm);
    dpsSubVect(L,((DPSsphere*)object->object)->position, ((DPSparticle*)p->data)->position);
    dpsProdScalVect(&v, L, Vn); //produto escalar de 2 vectores
    
    dpsProdScalVect(&tmp, L, L);
    disc = routsqr - tmp + v * v;

    if(disc < 0) continue;
      
    //teste da segunda zona de rejeicao
    t = v - sqrt(disc);
    if(t < 0 || t > (vm * (object->var1))) continue;
     
    dpsProdVect(C, Vn, L); 
    dpsNormalizeVect(C, C);
    dpsProdVect(S, Vn, C); 
        
    dpsMulVectConst(Vtmp, S, magdt / (t*t+object->var3));
    dpsAddVect(Vtmp, Vtmp, Vn);
    dpsLengthVect(&tmp, Vtmp);
    dpsMulVectConst(((DPSparticle*)p->data)->velocity , Vtmp, vm/tmp);
    
  } 
}


//definicao da accao AVOID para o objecto disco
void dpsAvoidDisc(DPSobject *object, GList* particles, DPSfloat dt){
  GList* p    = particles;
  DPSvector3f Pnext, Vtmp, Offset, S, Phit, Vn;
  DPSfloat rinsqr, routsqr, vm, magdt, rad, rin, rout,
           tmp, distold, distnew, nv, t;
  
  magdt = object->var2 * dt;
  rout = ((DPSdisc*)object->object)->radiusOut;
  rin  = ((DPSdisc*)object->object)->radiusIn;
  rinsqr  = rin * rin;
  routsqr = rout * rout; 
  
  for(;p; p=g_list_next(p)) { //para cada particula
    //verificar se a proxima posicao da particula
    //ja se encontra dentro do objecto
    //var1 = lookahead
    dpsMulVectConst(Vtmp,((DPSparticle*)p->data)->velocity , dt * (object->var1));
    dpsAddVect(Pnext,((DPSparticle*)p->data)->position, Vtmp);
    
    //position2 contem o vector normal ao plano (A,B,C da eq do plano)
    //distold e dist new : posicoes em relacao ao plano antes e depois
    // da iteraccao dist(n,plano)=n*p+D
    //distnace guarda -n*p,ou seja D da eq do plano
    dpsProdScalVect(&tmp,((DPSparticle*)p->data)->position, ((DPSdisc*)object->object)->position2);
    distold = tmp + ((DPSdisc*)object->object)->distance;
    dpsProdScalVect(&tmp, Pnext ,((DPSdisc*)object->object)->position2);
    distnew = tmp + ((DPSdisc*)object->object)->distance;
    

    if(distold * distnew >= 0) continue;
    
    dpsProdScalVect(&nv, ((DPSdisc*)object->object)->position2 ,((DPSparticle*)p->data)->velocity);     
    t = -1.0f*distold / nv;

    //actual interseccao
    dpsMulVectConst(Vtmp, ((DPSparticle*)p->data)->velocity, t);
    dpsAddVect(Phit, ((DPSparticle*)p->data)->position, Vtmp);
    //offset em relacao a origem no plano,p -origem
    dpsSubVect(Offset,Phit,((DPSdisc*)object->object)->position1);

    dpsSqrLengthVect(&rad,Offset);
    if(rad > routsqr || rad < rinsqr) continue;
  
    S[0] = Offset[0];
    S[1] = Offset[1];
    S[2] = Offset[2];
    dpsNormalizeVect(S, S);

    //modulo da velocidade da particula
    dpsLengthVect(&vm, ((DPSparticle*)p->data)->velocity );
    dpsDivVectConst(Vn, ((DPSparticle*)p->data)->velocity, vm);

    //calculo da vel. final
    dpsMulVectConst(Vtmp, S, magdt / (t*t+object->var3));
    dpsAddVect(Vtmp, Vtmp, Vn);
    dpsLengthVect(&tmp, Vtmp);
    dpsMulVectConst(((DPSparticle*)p->data)->velocity , Vtmp, vm/tmp);
    }
}

//funcao que calcula o AVOID com um rectangulo
void dpsAvoidRectangle(DPSobject *object, GList* particles, DPSfloat dt){
  GList* p    = particles;
  DPSvector3f S1,S2,S, Pnext, Vtmp, Vtmp1, Vtmp2, Phit, Offset, Vn, Vt, 
              Unorm, Vnorm, Uoffs, Voffs, Foffs, Goffs, Foffset;
  DPSfloat wx, wy, wz, xy, xz, ux, uy, uz, vx , vy, vz,
   distold, distnew, t, upos, vpos, nmag, det, n2, tmp, ulen, vlen, vm,
   gdistsqr, fdistsqr, udistsqr, vdistsqr, magdt;


  magdt = object->var2 * dt;
  //calcular a matrix inversa do plano da base
  ux = ((DPSrectangle*)object->object)->u[0];
  uy = ((DPSrectangle*)object->object)->u[1];
  uz = ((DPSrectangle*)object->object)->u[2];
    
  vx = ((DPSrectangle*)object->object)->v[0];
  vy = ((DPSrectangle*)object->object)->v[1];
  vz = ((DPSrectangle*)object->object)->v[2];
  
  //norma dos vectores U e V
  dpsLengthVect(&ulen, ((DPSrectangle*)object->object)->u);
  dpsLengthVect(&vlen, ((DPSrectangle*)object->object)->v);
  //normalizacao das bases
  dpsDivVectConst(Unorm, ((DPSrectangle*)object->object)->u, ulen);
  dpsDivVectConst(Vnorm, ((DPSrectangle*)object->object)->v, vlen);
  
  // w = u X v
  wx = uy*vz - uz*vy;
  wy = uz*vx - ux*vz;
  wz = ux*vy - uy*vx;
    
  det =1/(wz*ux*vy - wz*uy*vx - uz*wx*vy - ux*vz*wy + vz*wx*uy + uz*vx*wy);
  
  S1[0] = vy*wz - vz*wy;
  S1[1] = vz*wx - vx*wz;
  S1[2] = vx*wy - vy*wx;
  dpsMulVectConst(S1, S1, det);
  
  S2[0] = uy*wz - uz*wy;
  S2[1] = uz*wx - ux*wz;
  S2[2] = ux*wy - uy*wx;
  dpsMulVectConst(S2, S2, -1.0f*det);
  
  for(;p; p=g_list_next(p)) { //para cada particula
    dpsMulVectConst(Vtmp,((DPSparticle*)p->data)->velocity, dt*(object->var1));
    dpsAddVect(Pnext,((DPSparticle*)p->data)->position, Vtmp);
    
    //position2 contem o vector normal ao plano (A,B,C da eq do plano)
    //distold e dist new : posicoes em relacao ao plano antes e depois
    // da iteraccao dist(n,plano)=n*p+D
    //distnace guarda -n*p,ou seja D da eq do plano
    dpsProdScalVect(&tmp,((DPSparticle*)p->data)->position ,((DPSrectangle*)object->object)->position2);
    distold = tmp + ((DPSrectangle*)object->object)->distance;
    dpsProdScalVect(&tmp, Pnext ,((DPSrectangle*)object->object)->position2);
    distnew = tmp + ((DPSrectangle*)object->object)->distance;
  
    if(distold * distnew >= 0)     continue;
   
    dpsProdScalVect(&tmp,((DPSrectangle*)object->object)->position2,((DPSparticle*)p->data)->velocity);
    t = -1.0f*distold/tmp;
    
    //actual interseccao
    dpsMulVectConst(Vtmp, ((DPSparticle*)p->data)->velocity, t);
    dpsAddVect(Phit, ((DPSparticle*)p->data)->position, Vtmp);
    //offset em relacao a origem no plano,p -origem
    dpsSubVect(Offset,Phit,((DPSrectangle*)object->object)->position1);
    
    dpsProdScalVect(&upos, Offset, S1);  
    dpsProdScalVect(&vpos, Offset, S2);  
    
    if(upos < 0 || upos > 1 || vpos < 0 || vpos>1)  continue;

    //calcular as distancias entre os tres vertices
    dpsProdScalVect(Vtmp1, Unorm, Offset);
    dpsProdScalVect(Vtmp2, Unorm, Vtmp1);
    dpsSubVect(Uoffs, Vtmp2, Offset);
    dpsSqrLengthVect(&udistsqr, Uoffs);
    
    dpsProdScalVect(Vtmp1, Vnorm, Offset);
    dpsProdScalVect(Vtmp2, Vnorm, Vtmp1);
    dpsSubVect(Voffs, Vtmp2, Offset);
    dpsSqrLengthVect(&vdistsqr, Voffs);
    
    dpsAddVect(Vtmp1,((DPSrectangle*)object->object)->u , ((DPSrectangle*)object->object)->v);
    dpsSubVect(Foffset, Vtmp1, Offset);
    
    dpsProdScalVect(Vtmp1, Unorm, Foffset);
    dpsProdScalVect(Vtmp2, Unorm, Vtmp1);
    dpsSubVect(Foffs, Vtmp2, Foffset);
    dpsSqrLengthVect(&fdistsqr, Foffs);
    
    dpsProdScalVect(Vtmp1, Unorm, Foffset); //???Vnorm
    dpsProdScalVect(Vtmp2, Unorm, Vtmp1);  //??Vnorm
    dpsSubVect(Goffs, Vtmp2, Foffset);
    dpsSqrLengthVect(&gdistsqr, Goffs);
    
    //calcular o vector S
    if(udistsqr <= gdistsqr && udistsqr <= fdistsqr && udistsqr <= gdistsqr){
      S[0] = Uoffs[0];
      S[1] = Uoffs[1];
      S[2] = Uoffs[2];
    }else if(vdistsqr <= fdistsqr && vdistsqr <= gdistsqr){
            S[0] = Voffs[0];
            S[1] = Voffs[1];
            S[2] = Voffs[2];
          } else if (fdistsqr <= gdistsqr){
	          S[0] = Foffs[0];
                  S[1] = Foffs[1];
                  S[2] = Foffs[2]; 
	         }else {
		   S[0] = Goffs[0];
                   S[1] = Goffs[1];
                   S[2] = Goffs[2];
		 }
    
    

    dpsNormalizeVect(S, S);
    //calculo da vel. final
    dpsLengthVect(&vm, ((DPSparticle*)p->data)->velocity );
    dpsDivVectConst(Vn, ((DPSparticle*)p->data)->velocity, vm);

    //calculo da vel. final


    dpsMulVectConst(Vtmp, S, magdt / (t*t+object->var3));
    dpsAddVect(Vtmp, Vtmp, Vn);
    dpsLengthVect(&tmp, Vtmp);
    dpsMulVectConst(((DPSparticle*)p->data)->velocity , Vtmp, vm/tmp);
    }
}






//funcao que calcula o AVOID com um triangulo
void dpsAvoidTriangle(DPSobject *object, GList* particles, DPSfloat dt){
  GList* p    = particles;
  DPSvector3f S1,S2,S, Pnext, Vtmp, Vtmp1, Vtmp2, Phit, Offset, Vn, Vt, 
              Unorm, Vnorm, Uoffs, Voffs, Foffs, Goffs, Foffset, F, Fnorm;
  DPSfloat wx, wy, wz, xy, xz, ux, uy, uz, vx , vy, vz,
   distold, distnew, t, upos, vpos, nmag, det, n2, tmp, ulen, vlen, vm,
   gdistsqr, fdistsqr, udistsqr, vdistsqr, magdt;


  magdt = object->var2 * dt;
  //calcular a matrix inversa do plano da base
  ux = ((DPStriangle*)object->object)->u[0];
  uy = ((DPStriangle*)object->object)->u[1];
  uz = ((DPStriangle*)object->object)->u[2];
    
  vx = ((DPStriangle*)object->object)->v[0];
  vy = ((DPStriangle*)object->object)->v[1];
  vz = ((DPStriangle*)object->object)->v[2];
  
  //norma dos vectores U e V
  dpsLengthVect(&ulen, ((DPStriangle*)object->object)->u);
  dpsLengthVect(&vlen, ((DPStriangle*)object->object)->v);
  //normalizacao das bases
  dpsDivVectConst(Unorm, ((DPStriangle*)object->object)->u, ulen);
  dpsDivVectConst(Vnorm, ((DPStriangle*)object->object)->v, vlen);
  
  //F e'o terceiro(nao base) vertice do triangulo
  dpsSubVect(F, ((DPStriangle*)object->object)->u, ((DPStriangle*)object->object)->v);
  Fnorm[0] = F[0];
  Fnorm[1] = F[1];
  Fnorm[2] = F[2];
  dpsNormalizeVect(Fnorm, Fnorm);
 
  // w = u X v
  wx = uy*vz - uz*vy;
  wy = uz*vx - ux*vz;
  wz = ux*vy - uy*vx;
    
  det =1/(wz*ux*vy - wz*uy*vx - uz*wx*vy - ux*vz*wy + vz*wx*uy + uz*vx*wy);
  
  S1[0] = vy*wz - vz*wy;
  S1[1] = vz*wx - vx*wz;
  S1[2] = vx*wy - vy*wx;
  dpsMulVectConst(S1, S1, det);
  
  S2[0] = uy*wz - uz*wy;
  S2[1] = uz*wx - ux*wz;
  S2[2] = ux*wy - uy*wx;
  dpsMulVectConst(S2, S2, -1.0f*det);
  
  for(;p; p=g_list_next(p)) { //para cada particula
    dpsMulVectConst(Vtmp,((DPSparticle*)p->data)->velocity, dt*(object->var1));
    dpsAddVect(Pnext,((DPSparticle*)p->data)->position, Vtmp);
    
    //position2 contem o vector normal ao plano (A,B,C da eq do plano)
    //distold e dist new : posicoes em relacao ao plano antes e depois
    // da iteraccao dist(n,plano)=n*p+D
    //distnace guarda -n*p,ou seja D da eq do plano
    dpsProdScalVect(&tmp,((DPSparticle*)p->data)->position ,((DPStriangle*)object->object)->position2);
    distold = tmp + ((DPStriangle*)object->object)->distance;
    dpsProdScalVect(&tmp, Pnext ,((DPStriangle*)object->object)->position2);
    distnew = tmp + ((DPStriangle*)object->object)->distance;
  
    if(distold * distnew >= 0)     continue;
   
    dpsProdScalVect(&tmp,((DPStriangle*)object->object)->position2,((DPSparticle*)p->data)->velocity);
    t = -1.0f*distold/tmp;
    
    //actual interseccao
    dpsMulVectConst(Vtmp, ((DPSparticle*)p->data)->velocity, t);
    dpsAddVect(Phit, ((DPSparticle*)p->data)->position, Vtmp);
    //offset em relacao a origem no plano,p -origem
    dpsSubVect(Offset,Phit,((DPStriangle*)object->object)->position1);
    
    dpsProdScalVect(&upos, Offset, S1);  
    dpsProdScalVect(&vpos, Offset, S2);  
    
    if(upos < 0 || vpos < 0 || (upos+vpos) > 1)  continue;

    //calcular as distancias entre os tres vertices
    dpsProdScalVect(Vtmp1, Unorm, Offset);
    dpsProdScalVect(Vtmp2, Unorm, Vtmp1);
    dpsSubVect(Uoffs, Vtmp2, Offset);
    dpsSqrLengthVect(&udistsqr, Uoffs);
    
    dpsProdScalVect(Vtmp1, Vnorm, Offset);
    dpsProdScalVect(Vtmp2, Vnorm, Vtmp1);
    dpsSubVect(Voffs, Vtmp2, Offset);
    dpsSqrLengthVect(&vdistsqr, Voffs);
    
    dpsSubVect(Foffset, Offset, ((DPStriangle*)object->object)->u);
    
    dpsProdScalVect(Vtmp1, Fnorm, Foffset);
    dpsProdScalVect(Vtmp2, Fnorm, Vtmp1);
    dpsSubVect(Foffs, Vtmp2, Foffset);
    dpsSqrLengthVect(&fdistsqr, Foffs);
    
    //calcular o vector S
    if(udistsqr <= vdistsqr && udistsqr <= fdistsqr){
      S[0] = Uoffs[0];
      S[1] = Uoffs[1];
      S[2] = Uoffs[2];
    }else if(vdistsqr <= fdistsqr){
            S[0] = Voffs[0];
            S[1] = Voffs[1];
            S[2] = Voffs[2];
          }else{
	    S[0] = Foffs[0];
            S[1] = Foffs[1];
            S[2] = Foffs[2];
	  }
    dpsNormalizeVect(S, S);
    
    //calculo da vel. final
    dpsLengthVect(&vm, ((DPSparticle*)p->data)->velocity );
    dpsDivVectConst(Vn, ((DPSparticle*)p->data)->velocity, vm);
    //calculo da vel. final
    dpsMulVectConst(Vtmp, S, magdt / (t*t+object->var3));
    dpsAddVect(Vtmp, Vtmp, Vn);
    dpsLengthVect(&tmp, Vtmp);
    dpsMulVectConst(((DPSparticle*)p->data)->velocity , Vtmp, vm/tmp);
    }
}
//funcao que evita o colisao de um particula com o plano
void dpsAvoidPlane(DPSobject *object, GList* particles, DPSfloat dt){
  GList* p    = particles;
  DPSvector3f Vtmp, Vn;
  DPSfloat magdt, tmp, dist, vm;
  
  
  magdt = object->var2 * dt;
  
  if(object->var1 < DPS_MAXFLOAT){ //lookahead < MAXFLOAT
      for(;p; p=g_list_next(p)) { //para cada particula
        //position2 contem o vector normal ao plano (A,B,C da eq do plano)
        //distold e dist new : posicoes em relacao ao plano antes e depois
        // da iteraccao dist(n,plano)=n*p+D
        //dist guarda -n*p,ou seja D da eq do plano      
        dpsProdScalVect(&tmp,((DPSparticle*)p->data)->position ,((DPSplane*)object->object)->position2);
        dist = tmp + ((DPSplane*)object->object)->distance;
	if(dist < object->var1){
	 //calculo da vel. final
         dpsLengthVect(&vm,  ((DPSparticle*)p->data)->velocity);
         dpsDivVectConst(Vn, ((DPSparticle*)p->data)->velocity, vm);
         //calculo da vel. final
         dpsMulVectConst(Vtmp,((DPSplane*)object->object)->position2 , magdt / (dist*dist+object->var3));
         dpsAddVect(Vtmp, Vtmp, Vn);
         dpsLengthVect(&tmp, Vtmp);
         dpsMulVectConst(((DPSparticle*)p->data)->velocity , Vtmp, vm/tmp);
	}	
      }
  }else{
    for(;p; p=g_list_next(p)) { //para cada particula
        dpsProdScalVect(&tmp,((DPSparticle*)p->data)->position ,((DPSplane*)object->object)->position2);
        dist = tmp + ((DPSplane*)object->object)->distance;
	//calculo da vel. final
        dpsLengthVect(&vm, ((DPSparticle*)p->data)->velocity);
        dpsDivVectConst(Vn, ((DPSparticle*)p->data)->velocity, vm);
        //calculo da vel. final
        dpsMulVectConst(Vtmp,((DPSplane*)object->object)->position2 , magdt / (dist*dist+object->var3));
        dpsAddVect(Vtmp, Vtmp, Vn);
        dpsLengthVect(&tmp, Vtmp);
        dpsMulVectConst(((DPSparticle*)p->data)->velocity , Vtmp, vm/tmp);
     }
  }
}
    
    
void dpsKillSphere(DPSobject *object, GList** particles, DPSsets sets){
  GList *t, *p;
  
  for(p = *particles; p;  p = g_list_next(p)) {
    if(isWithinSphere(((DPSparticle*)p->data)->position[0], 
                    ((DPSparticle*)p->data)->position[1], 
		    ((DPSparticle*)p->data)->position[2], 
		    (DPSsphere*)object->object)){

      // executar a funcao die
      if(((DPSparticle*)p->data)->die)  
        ((DPSparticle*)p->data)->die((DPSparticle*)p, 
                                      sets.particles,
                                      sets.forces,
                                      sets.objects);


  		
      t = p;
      p = g_list_next(p);
      *particles = g_list_remove_link(*particles, t);		

    }
  }
}



void dpsKillCube(DPSobject *object, GList** particles, DPSsets sets){
  GList *t, *p;
  
  for(p = *particles; p;  p = g_list_next(p)) {
    if(isWithinCube(((DPSparticle*)p->data)->position[0], 
                    ((DPSparticle*)p->data)->position[1], 
		    ((DPSparticle*)p->data)->position[2], 
		    (DPScube*)object->object)){
  		
      // executar a funcao die
      if(((DPSparticle*)p->data)->die)  
        ((DPSparticle*)p->data)->die((DPSparticle*)p, 
                                      sets.particles,
                                      sets.forces,
                                      sets.objects);

      t = p;
      p = g_list_next(p);
      *particles = g_list_remove_link(*particles, t);		


    }
  }
}





void dpsKillPlane(DPSobject *object, GList** particles, DPSfloat dt, DPSsets sets){
  GList *t, *p;
  DPSvector3f Pnext, Vtmp;
  
  for(p = *particles; p;  p = g_list_next(p)) {
    
    dpsMulVectConst(Vtmp,((DPSparticle*)p->data)->velocity, dt);
    dpsAddVect(Pnext,((DPSparticle*)p->data)->position, Vtmp);
    
    if(isWithinPlane(((DPSparticle*)p->data)->position[0], 
                     ((DPSparticle*)p->data)->position[1], 
		     ((DPSparticle*)p->data)->position[2], 
		     Pnext[0], Pnext[1], Pnext[2],
		     (DPSplane*)object->object)){

      // executar a funcao die
      if(((DPSparticle*)p->data)->die)  
        ((DPSparticle*)p->data)->die((DPSparticle*)p, 
                                      sets.particles,
                                      sets.forces,
                                      sets.objects);

      t = p;
      p = g_list_next(p);
      *particles = g_list_remove_link(*particles, t);		


    }
  }
}

void dpsKillDisc(DPSobject *object, GList** particles, DPSfloat dt, DPSsets sets){
  GList *t, *p;
  DPSvector3f Pnext, Vtmp;
  
  for(p = *particles; p;  p = g_list_next(p)) {
    
    dpsMulVectConst(Vtmp,((DPSparticle*)p->data)->velocity, dt);
    dpsAddVect(Pnext,((DPSparticle*)p->data)->position, Vtmp);
    
    if(isWithinDisc(((DPSparticle*)p->data)->position[0], 
                    ((DPSparticle*)p->data)->position[1], 
		    ((DPSparticle*)p->data)->position[2], 
		    Pnext[0], Pnext[1], Pnext[2],
		    ((DPSparticle*)p->data)->velocity[0], 
                    ((DPSparticle*)p->data)->velocity[1], 
		    ((DPSparticle*)p->data)->velocity[2], 
		    (DPSdisc*)object->object)){

      // executar a funcao die
      if(((DPSparticle*)p->data)->die)  
        ((DPSparticle*)p->data)->die((DPSparticle*)p, 
                                      sets.particles,
                                      sets.forces,
                                      sets.objects);


      t = p;
      p = g_list_next(p);
      *particles = g_list_remove_link(*particles, t);		

    }
  }
}









void dpsKillTriangle(DPSobject *object, GList** particles, DPSfloat dt, DPSsets sets){
  GList *t, *p;
  DPSvector3f Pnext, Vtmp;
  
  for(p = *particles; p;  p = g_list_next(p)) {
    
    dpsMulVectConst(Vtmp,((DPSparticle*)p->data)->velocity, dt);
    dpsAddVect(Pnext,((DPSparticle*)p->data)->position, Vtmp);
    
    if(isWithinTriangle(((DPSparticle*)p->data)->position[0], 
                     ((DPSparticle*)p->data)->position[1], 
		     ((DPSparticle*)p->data)->position[2], 
		     Pnext[0], Pnext[1], Pnext[2],
		     (DPStriangle*)object->object)){

      // executar a funcao die
      if(((DPSparticle*)p->data)->die)  
        ((DPSparticle*)p->data)->die((DPSparticle*)p, 
                                      sets.particles,
                                      sets.forces,
                                      sets.objects);

      t = p;
      p = g_list_next(p);
      *particles = g_list_remove_link(*particles, t);		

    }
  }
}


void dpsKillRectangle(DPSobject *object, GList** particles, DPSfloat dt, DPSsets sets) {
  GList *t, *p;
  DPSvector3f Pnext, Vtmp;

// isto esta errado
  
  for(p = *particles; p;  p = g_list_next(p)) {
    
    dpsMulVectConst(Vtmp,((DPSparticle*)p->data)->velocity, dt);
    dpsAddVect(Pnext,((DPSparticle*)p->data)->position, Vtmp);
    

      // executar a funcao die
      if(((DPSparticle*)p->data)->die)  
        ((DPSparticle*)p->data)->die((DPSparticle*)p, 
                                      sets.particles,
                                      sets.forces,
                                      sets.objects);

      t = p;
      p = g_list_next(p);
      *particles = g_list_remove_link(*particles, t);		


    }
}






void dpsKillCone(DPSobject *object, GList** particles, DPSsets sets){
  GList *t, *p;
  
  for(p = *particles; p;  p = g_list_next(p)) {
    if(isWithinCone(((DPSparticle*)p->data)->position[0], 
                    ((DPSparticle*)p->data)->position[1], 
		    ((DPSparticle*)p->data)->position[2], 
		    (DPScone*)object->object)){

      // executar a funcao die
      if(((DPSparticle*)p->data)->die)  
        ((DPSparticle*)p->data)->die((DPSparticle*)p, 
                                      sets.particles,
                                      sets.forces,
                                      sets.objects);

  		
      t = p;
      p = g_list_next(p);
      *particles = g_list_remove_link(*particles, t);		


    }
  }
}


void dpsKillCylinder(DPSobject *object, GList** particles, DPSsets sets){
  GList *t, *p;
  
  for(p = *particles; p;  p = g_list_next(p)) {
    if(isWithinCylinder(((DPSparticle*)p->data)->position[0], 
                        ((DPSparticle*)p->data)->position[1], 
		        ((DPSparticle*)p->data)->position[2], 
		        (DPScylinder*)object->object)){

      // executar a funcao die
      if(((DPSparticle*)p->data)->die)  
        ((DPSparticle*)p->data)->die((DPSparticle*)p, 
                                      sets.particles,
                                      sets.forces,
                                      sets.objects);

  		
      t = p;
      p = g_list_next(p);
      *particles = g_list_remove_link(*particles, t);		


    }
  }
}
/******************************************************************************/
/**** LOCAL ONDE SAO DECLARADOS OS GERADORES DE PARTICULAS               *****/ 
/**** OS PARAMETROS SAO OS SEGUINTES:                                    *****/
/**** VAR1 : RATIO OU TAXA DE GERACAO					******/
/**** VAR2 : MAGNITUDE DA VEL.						******/
/**** VAR3 : STDEV OU SIGMA PARA UMA DISTRIBUICAO NORMAL                  ****/
/*****************************************************************************/
void dpsSourceSphere(DPSobject *object, GList** particles, DPSstate *state){
  DPSvector3f pos, vel = {0.0, 0.0, 0.0};
  DPSulong i;

  for(i=0; i< floor(object->counter); i++) {   
    // gerar posicao dentro do objecto
    dpsGenSpherePos(pos, 
      ((DPSsphere*)object->object)->position[0],
      ((DPSsphere*)object->object)->position[1],
      ((DPSsphere*)object->object)->position[2],
      ((DPSsphere*)object->object)->radiusIn,
      ((DPSsphere*)object->object)->radiusOut);
    

    // gerar o vector velocidade com que sai do objecto
    // e normalizalo a var2 com desvio var32
    dpsSubVect(vel, pos, ((DPSsphere*)object->object)->position);
    dpsNormalizeVect(vel, vel);
    dpsMulVectConst(vel, vel, dpsRandomNormalDist(object->var2, object->var3));


    *particles = g_list_append(*particles, 
           dpsNewParticle10v(pos, vel, 
                    state->__DPS_STATE_MASS,
                    state->__DPS_STATE_AGE,
                    state->__DPS_STATE_COLOR,
                    state->__DPS_STATE_COLOR_DEV,
                    state->__DPS_STATE_ROLL,
                    state->__DPS_STATE_MESH,
                    state->__DPS_STATE_DIE,
                    state->__DPS_STATE_COLLIDE));
  }                    
  object->counter = (object->counter+object->var1) - floor(object->counter);
}



void dpsSourceDisc(DPSobject *object, GList** particles, DPSstate *state){
  DPSvector3f pos, vel;
  DPSulong i;

  for(i=0; i< floor(object->counter); i++) {   
    // gerar posicao dentro do objecto
    dpsGenDiscPos(pos, 
      ((DPSdisc*)object->object)->position1[0],
      ((DPSdisc*)object->object)->position1[1],
      ((DPSdisc*)object->object)->position1[2],
      ((DPSdisc*)object->object)->position2[0],
      ((DPSdisc*)object->object)->position2[1],
      ((DPSdisc*)object->object)->position2[2],
      ((DPSdisc*)object->object)->radiusIn,
      ((DPSdisc*)object->object)->radiusOut);
    

    // gerar o vector velocidade com que sai do objecto
    // e normalizalo a var2 com desvio var3
    dpsNormalizeVect(vel, ((DPSdisc*)object->object)->position2);
    dpsMulVectConst(vel, vel, dpsRandomNormalDist(object->var2, object->var3));


    *particles = g_list_append(*particles, 
           dpsNewParticle10v(pos, vel, 
                    state->__DPS_STATE_MASS,
                    state->__DPS_STATE_AGE,
                    state->__DPS_STATE_COLOR,
                    state->__DPS_STATE_COLOR_DEV,
                    state->__DPS_STATE_ROLL,
                    state->__DPS_STATE_MESH,
                    state->__DPS_STATE_DIE,
                    state->__DPS_STATE_COLLIDE));
  }                    
  object->counter = (object->counter+object->var1) - floor(object->counter);
}


void dpsSourceCone(DPSobject *object, GList** particles, DPSstate *state){
  DPSvector3f pos, vel;
  DPSulong i;

  for(i=0; i< floor(object->counter); i++) {   
    // gerar posicao dentro do objecto
    dpsGenConePos(pos, 
      ((DPScone*)object->object)->position1[0],
      ((DPScone*)object->object)->position1[1],
      ((DPScone*)object->object)->position1[2],
      ((DPScone*)object->object)->tmp[0],
      ((DPScone*)object->object)->tmp[1],
      ((DPScone*)object->object)->tmp[2],
      ((DPScone*)object->object)->radiusIn,
      ((DPScone*)object->object)->radiusOut);
    

    // gerar o vector velocidade com que sai do objecto
    // e normalizalo a var2 com desvio var3
    dpsSubVect(vel, pos, ((DPScone*)object->object)->position1);
    dpsNormalizeVect(vel, vel);
    dpsMulVectConst(vel, vel, dpsRandomNormalDist(object->var2, object->var3));


    *particles = g_list_append(*particles, 
           dpsNewParticle10v(pos, vel, 
                    state->__DPS_STATE_MASS,
                    state->__DPS_STATE_AGE,
                    state->__DPS_STATE_COLOR,
                    state->__DPS_STATE_COLOR_DEV,
                    state->__DPS_STATE_ROLL,
                    state->__DPS_STATE_MESH,
                    state->__DPS_STATE_DIE,
                    state->__DPS_STATE_COLLIDE));
  }                    
  object->counter = (object->counter+object->var1) - floor(object->counter);
}



void dpsSourceCylinder(DPSobject *object, GList** particles, DPSstate *state){
  DPSvector3f pos, vel;
  DPSulong i;

  for(i=0; i< floor(object->counter); i++) {   
    // gerar posicao dentro do objecto
    dpsGenConePos(pos, 
      ((DPScylinder*)object->object)->position1[0],
      ((DPScylinder*)object->object)->position1[1],
      ((DPScylinder*)object->object)->position1[2],
      ((DPScylinder*)object->object)->position2[0],
      ((DPScylinder*)object->object)->position2[1],
      ((DPScylinder*)object->object)->position2[2],
      ((DPScylinder*)object->object)->radiusIn,
      ((DPScylinder*)object->object)->radiusOut);
    

    // gerar o vector velocidade com que sai do objecto
    // e normalizalo a var2 com desvio var3
    dpsSubVect(vel, pos, ((DPScylinder*)object->object)->tmp);
    dpsNormalizeVect(vel, vel);
    dpsMulVectConst(vel, vel, dpsRandomNormalDist(object->var2, object->var3));


    *particles = g_list_append(*particles, 
           dpsNewParticle10v(pos, vel, 
                    state->__DPS_STATE_MASS,
                    state->__DPS_STATE_AGE,
                    state->__DPS_STATE_COLOR,
                    state->__DPS_STATE_COLOR_DEV,
                    state->__DPS_STATE_ROLL,
                    state->__DPS_STATE_MESH,
                    state->__DPS_STATE_DIE,
                    state->__DPS_STATE_COLLIDE));
  }                    
  object->counter = (object->counter+object->var1) - floor(object->counter);
}




void dpsSourceRectangle(DPSobject *object, GList** particles, DPSstate *state){
  DPSvector3f pos, u, v, vel = {0.0 , 0.0, 0.0};
  DPSulong i;

  for(i=0; i< floor(object->counter); i++) {   
    // gerar posicao dentro do objecto
    dpsGenRectanglePos(pos, 
      ((DPSrectangle*)object->object)->position1[0],
      ((DPSrectangle*)object->object)->position1[1],
      ((DPSrectangle*)object->object)->position1[2],
      ((DPSrectangle*)object->object)->u[0],
      ((DPSrectangle*)object->object)->u[1],
      ((DPSrectangle*)object->object)->u[2],
      ((DPSrectangle*)object->object)->v[0],
      ((DPSrectangle*)object->object)->v[1],
      ((DPSrectangle*)object->object)->v[2]);
    

    // gerar o vector velocidade com que sai do objecto
    // e normalizando a var2 com desvio var3
    dpsAddVect(vel, vel, ((DPSrectangle*)object->object)->position2);
    dpsNormalizeVect(vel, vel);
    dpsMulVectConst(vel, vel, dpsRandomNormalDist(object->var2, object->var3));


    *particles = g_list_append(*particles, 
           dpsNewParticle10v(pos, vel, 
                    state->__DPS_STATE_MASS,
                    state->__DPS_STATE_AGE,
                    state->__DPS_STATE_COLOR,
                    state->__DPS_STATE_COLOR_DEV,
                    state->__DPS_STATE_ROLL,
                    state->__DPS_STATE_MESH,
                    state->__DPS_STATE_DIE,
                    state->__DPS_STATE_COLLIDE));
  }                    
  object->counter = (object->counter+object->var1) - floor(object->counter);
}


void dpsSourceTriangle(DPSobject *object, GList** particles, DPSstate *state){
  DPSvector3f pos, u, v, vel={0.0, 0.0, 0.0};
  DPSulong i;

  for(i=0; i< floor(object->counter); i++) {   
    // gerar posicao dentro do objecto
    dpsGenTrianglePos(pos, 
      ((DPStriangle*)object->object)->position1[0],
      ((DPStriangle*)object->object)->position1[1],
      ((DPStriangle*)object->object)->position1[2],
      ((DPStriangle*)object->object)->u[0],
      ((DPStriangle*)object->object)->u[1],
      ((DPStriangle*)object->object)->u[2],
      ((DPStriangle*)object->object)->v[0],
      ((DPStriangle*)object->object)->v[1],
      ((DPStriangle*)object->object)->v[2]);
    

    // gerar o vector velocidade com que sai do objecto
    // e normalizalo a var2 com desvio var3
    dpsAddVect(vel, vel, ((DPStriangle*)object->object)->position2);
    dpsNormalizeVect(vel, vel);
    dpsMulVectConst(vel, vel, dpsRandomNormalDist(object->var2, object->var3));


    *particles = g_list_append(*particles, 
           dpsNewParticle10v(pos, vel, 
                    state->__DPS_STATE_MASS,
                    state->__DPS_STATE_AGE,
                    state->__DPS_STATE_COLOR,
                    state->__DPS_STATE_COLOR_DEV,
                    state->__DPS_STATE_ROLL,
                    state->__DPS_STATE_MESH,
                    state->__DPS_STATE_DIE,
                    state->__DPS_STATE_COLLIDE));
  }                    
  object->counter = (object->counter+object->var1) - floor(object->counter);
}

void dpsSourceCube(DPSobject *object, GList** particles, DPSstate *state){
  DPSvector3f pos, u, v, vel;
  DPSulong i;

  for(i=0; i< floor(object->counter); i++) {   
    // gerar posicao dentro do objecto
    dpsGenCubePos(pos, 
      ((DPScube*)object->object)->position1[0],
      ((DPScube*)object->object)->position1[1],
      ((DPScube*)object->object)->position1[2],
      ((DPScube*)object->object)->position2[0],
      ((DPScube*)object->object)->position2[1],
      ((DPScube*)object->object)->position2[2]);
    

    // gerar o vector velocidade com que sai do objecto
    // e normalizalo a var2 com desvio var3
    dpsSubVect(vel, pos, ((DPScube*)object->object)->position2);
    dpsNormalizeVect(vel, vel);
    dpsMulVectConst(vel, vel, dpsRandomNormalDist(object->var2, object->var3));


    *particles = g_list_append(*particles, 
           dpsNewParticle10v(pos, vel, 
                    state->__DPS_STATE_MASS,
                    state->__DPS_STATE_AGE,
                    state->__DPS_STATE_COLOR,
                    state->__DPS_STATE_COLOR_DEV,
                    state->__DPS_STATE_ROLL,
                    state->__DPS_STATE_MESH,
                    state->__DPS_STATE_DIE,
                    state->__DPS_STATE_COLLIDE));
  }                    
  object->counter = (object->counter+object->var1) - floor(object->counter);
}

void dpsSourcePlane(DPSobject *object, GList** particles, DPSstate *state){
  DPSvector3f pos, u, v;
  DPSobject* newObject; 
  
  pos[0] = ((DPSplane*)object->object)->position1[0];
  pos[1] = ((DPSplane*)object->object)->position1[1];
  pos[2] = ((DPSplane*)object->object)->position1[2];
  
  u[0] = ((DPSplane*)object->object)->position2[0];
  u[1] = ((DPSplane*)object->object)->position2[1];
  u[2] = ((DPSplane*)object->object)->position2[2];
  
  v[0] = u[0] - pos[0];
  v[1] = u[1] - pos[1];
  v[1] = u[2] - pos[2];
  
  //vamos definir um plano ap partir de um rectangulo com 
  // a mesma orientcao do plano mas com uma dimensao muito grande
  // para o tornar infinito
  
  dpsMulVectConst(u, u, DPS_MAXFLOAT); // torna o rectangulo infinito
  dpsMulVectConst(v, v, DPS_MAXFLOAT); // torna o rectangulo infinito
  
  newObject = dpsNewRectangle13f(pos[0], pos[1], pos[2], 
                                 u[0], u[1], u[2],
			         v[0], v[1], v[2],
  		                 object->action, 
				 object->var1, 
			         object->var2, 
			         object->var3);
					 
  dpsSourceRectangle(newObject, particles, state);					 

}  





/******************************************************************************/
/**** LOCAL ONDE ACABAM OS GERADORES DE PARTICULAS               *****/ 
/*****************************************************************************/














/*a ordem dos tres primeiros argumentos sera no caso da accao
 * avoid:   var1:look_ahead (1.0)  <= valores tipicos
 *          var2:magnitude  (20)
 *          var3:epsilon    (0.2)
 *
 *a ordem dos tres primeiros argumentos sera no caso da accao
 * collide: var1:one_minus_friction
 *          var2:resilience (0) fica colado a supeficie (1) sai com a mesma vel
 *          var3:cut_off_sqr
 *
*/



void dpsApplyObject(DPSobject *object, GList** particles, DPSstate *state, DPSsets sets) {
  switch(object->type) {
    case DPS_SPHERE:    // Inicio da esfera
      switch(object->action) {
        case DPS_AVOID   : dpsAvoidSphere(object, *particles, state->__DPS_STATE_DT); break;
	case DPS_COLLIDE : dpsCollideSphere(object, *particles, state->__DPS_STATE_DT, sets);break;  
	case DPS_KILL    : dpsKillSphere(object, particles, sets); break; 
	case DPS_SOURCE  : dpsSourceSphere(object, particles, state); break; 
      } //acaba aqui a esfera
    break;
    case DPS_RECTANGLE: 
      switch(object->action) {
        case DPS_AVOID   : dpsAvoidRectangle(object, *particles, state->__DPS_STATE_DT); break;
	case DPS_COLLIDE : dpsCollideRectangle(object, *particles, state->__DPS_STATE_DT, sets);break; 
	case DPS_KILL    : dpsKillRectangle(object, particles, state->__DPS_STATE_DT, sets);break;  
	case DPS_SOURCE  : dpsSourceRectangle(object, particles, state); break; 
      } //acaba aqui o rectangulo
    break;      
    case DPS_PLANE: 
      switch(object->action) {
        case DPS_AVOID   : dpsAvoidPlane(object, *particles, state->__DPS_STATE_DT);  break;
	case DPS_COLLIDE : dpsCollidePlane(object, *particles, state->__DPS_STATE_DT, sets);break; 
	case DPS_KILL    : dpsKillPlane(object, particles, state->__DPS_STATE_DT, sets);break; 
	case DPS_SOURCE  : break; 
      } //acaba aqui o plano
    break;
    case DPS_TRIANGLE: 
      switch(object->action) {
        case DPS_AVOID   : dpsAvoidTriangle(object, *particles, state->__DPS_STATE_DT); break;
	case DPS_COLLIDE : dpsCollideTriangle(object, *particles, state->__DPS_STATE_DT, sets);break; 
	case DPS_KILL    : dpsKillTriangle(object, particles, state->__DPS_STATE_DT, sets);break; 
	case DPS_SOURCE  : dpsSourceTriangle(object, particles, state); break; 
      } //acaba aqui o triangulo
    break;            
    case DPS_DISC: 
      switch(object->action) {
        case DPS_AVOID   : dpsAvoidDisc(object, *particles, state->__DPS_STATE_DT); break;
	case DPS_COLLIDE : dpsCollideDisc(object, *particles, state->__DPS_STATE_DT, sets);break;  
	case DPS_KILL    : dpsKillDisc(object, particles, state->__DPS_STATE_DT, sets);break; 
	case DPS_SOURCE  : dpsSourceDisc(object, particles, state); break; 
      } //acaba aqui o disco
    break;      
    case DPS_CONE: 
      switch(object->action) {
        case DPS_AVOID   : break;
	case DPS_COLLIDE : break;  
	case DPS_KILL    : dpsKillCone(object, particles, sets);break; 
	case DPS_SOURCE  : dpsSourceCone(object, particles, state); break; 
      } //acaba aqui o cone
    break;      
    case DPS_CYLINDER: 
      switch(object->action) {
        case DPS_AVOID   : break;
	case DPS_COLLIDE : break;  
	case DPS_KILL    : dpsKillCylinder(object, particles, sets);break; 
	case DPS_SOURCE  : dpsSourceCylinder(object, particles, state); break; 
      } //acaba aqui o CILINDRO
    break;      
   case DPS_CUBE: 
      switch(object->action) {
        case DPS_AVOID   : break;
	case DPS_COLLIDE : break;  
	case DPS_KILL    : dpsKillCube(object, particles, sets); break; 
	case DPS_SOURCE  : dpsSourceCube(object, particles, state); break; 
      } //acaba aqui o CILINDRO
    break;      
  }
}


